//------------------------------------
float4x4 LightWorldViewProjection;
float FarPlaneMinusNearPlane; // Distance of the far plane minus distance of the near plane.
texture g_txDepthMap;
texture g_txLeftMap;
texture g_txRightMap;
texture g_txTempMap;			//Map from DefaultPass
texture g_txSourceMap;			
texture g_txShadowMap;			//Final FilteredShadowMap
float2	depthMapPos;
float	depthMapScale;
float	errorThreshold;
float invHalfScreenWidth;
float invHalfScreenHeight;
float invMaxShadowMaps;
float4x4 refProjection;


//------------------------------------
struct VS_OUTPUT_SHADOWMAP
{
    float4 Position     : POSITION;    
    float3 Position2D    : TEXCOORD0;
};

struct PS_OUTPUT
{
    float4 diffuse : COLOR0;
};

struct vertexInputSDM {
    float3 position				: POSITION;
    float4 texCoord				: TEXCOORD0;
};

struct vertexOutputSDM {
    float4 HPosition 			: POSITION;
    float4 texCoord				: TEXCOORD0;
};

struct pixelOutputSDM
{
	float4 color				: COLOR;
};

struct pixelOutputCDM
{
	float4 color				: COLOR;
	float  depth				: DEPTH;
};

//Vertex definition for Copy
//Vertex definition for Reprojection
struct vertexInputCP {
    float3 position				: POSITION;
    float4 texCoord				: TEXCOORD0;
};

struct vertexOutputCP {
    float4 position 			: POSITION;
    float4 texCoord				: TEXCOORD0;
};

struct vertexInputRP {
    float3 position				: POSITION;
    float4 texCoord				: TEXCOORD1;
};

struct vertexOutputRP {
    float4 position 			: POSITION;
    float4 texCoord				: TEXCOORD1;
};

struct pixelOutputRP
{
	float4 color				: COLOR;
};


//------------------------------------
#define SAMPLER_LINEAR(g_samplerMap, g_txMap);	\
	sampler2D g_samplerMap = sampler_state {	\
    Texture = <g_txMap>;						\
    MinFilter = Linear;							\
    MagFilter = Linear;							\
    MipFilter = Linear;							\
    AddressU  = CLAMP;							\
    AddressV  = CLAMP;							\
};

SAMPLER_LINEAR(g_samplerShowDepthMap,	g_txDepthMap);
SAMPLER_LINEAR(g_samplerLeftDepthMap,	g_txLeftMap);
SAMPLER_LINEAR(g_samplerRightDepthMap,	g_txRightMap);
SAMPLER_LINEAR(g_samplerTempMap, g_txTempMap);
SAMPLER_LINEAR(g_samplerSourceMap, g_txSourceMap);
SAMPLER_LINEAR(g_samplerShadowMap, g_txShadowMap);

//------------------------------------
VS_OUTPUT_SHADOWMAP ShadowMapVertexShader( float4 inPos : POSITION)
{
    VS_OUTPUT_SHADOWMAP Output = (VS_OUTPUT_SHADOWMAP)0;
    
    Output.Position = mul(float4(inPos.xyz, 1.0f), LightWorldViewProjection);
    Output.Position2D = Output.Position;
    
    return Output;    
}

PS_OUTPUT ShadowMapPixelShader(VS_OUTPUT_SHADOWMAP IN)
{
    PS_OUTPUT Output = (PS_OUTPUT)0;
    Output.diffuse = (IN.Position2D.z/(FarPlaneMinusNearPlane * 0.5 ));
    return Output;
}

vertexOutputRP ClearDepthMapVS(vertexInputRP IN) {
	vertexOutputRP OUT;
	
	OUT.position = float4(IN.position.xy, 1, 1.0f);
	OUT.texCoord = IN.texCoord;
	
	return OUT;
}

pixelOutputRP ClearDepthMapPS(vertexOutputRP IN) {
	pixelOutputRP OUT;
	
	OUT.color = float4(1, 1, 1, 1);
	return OUT;
}

vertexOutputCP CopyVS(vertexInputCP IN) {
	vertexOutputCP OUT;
	
	OUT.position = float4(IN.position.xyz, 1.0f);
	OUT.texCoord = IN.texCoord;
	
	return OUT;
}

pixelOutputRP CopyPS(vertexOutputCP IN) {
	pixelOutputRP OUT;
	
	OUT.color = float4(tex2D(g_samplerTempMap, IN.texCoord.xy).xyz, 1);
	return OUT;
}

vertexOutputRP ReprojectionVS(vertexInputRP IN) {
	vertexOutputRP OUT;
	
	OUT.position = mul(float4(IN.position.xyz, 1.0f), refProjection);
	float4 sidePoint = mul(float4(IN.position.xyz, 1.0f), LightWorldViewProjection);
	
	OUT.texCoord = sidePoint;
	return OUT;
}

pixelOutputRP ReprojectionPS(vertexOutputRP IN) {
	pixelOutputRP OUT;
	
	float2 coord;
	coord.x = (IN.texCoord.x/IN.texCoord.w)*0.5f+0.5f + invHalfScreenWidth;
	coord.y = (-IN.texCoord.x/IN.texCoord.w)*0.5f+0.5f + invHalfScreenHeight;
	
	OUT.color = float4(tex2D(g_samplerSourceMap, coord.xy).xxx,1);
	
	return OUT;
}

vertexOutputSDM ShowDepthMapsVS(vertexInputSDM IN) {
	vertexOutputSDM OUT;
	float3 temp = float3(IN.position)*depthMapScale + float3(depthMapPos.xy,0);
	OUT.HPosition = float4(temp.xyz,1);
	OUT.texCoord = IN.texCoord;
	return OUT;
}

pixelOutputSDM ShowDepthMapsPS(vertexOutputSDM IN) {
	pixelOutputSDM OUT;
	OUT.color = float4(tex2D(g_samplerShowDepthMap, IN.texCoord.xy).xxx,1);
	return OUT;
}

vertexOutputSDM CompareDepthMapsVS(vertexInputSDM IN) {
	vertexOutputSDM OUT;
	OUT.HPosition = float4(IN.position.xyz, 1);
	OUT.texCoord = IN.texCoord;
	return OUT;
}

pixelOutputCDM CompareDepthMapsPS(vertexOutputSDM IN) {
	pixelOutputCDM OUT;
	float4 color = float4(0, 0, 0, 0);
	float z = 1.0f;
	float difference = abs(tex2D(g_samplerLeftDepthMap, IN.texCoord.xy).x - tex2D(g_samplerRightDepthMap, IN.texCoord.xy).x);
	if(difference>errorThreshold) {
		color = float4(1, 1, 1, 0);
		z = 0.3f;
	}
	OUT.color = color;
	OUT.depth = z;
	return OUT;
}

PS_OUTPUT CombinePS(vertexOutputCP IN) {
	PS_OUTPUT OUT = (PS_OUTPUT) 0;
	
	float nbShadows = tex2D(g_samplerShadowMap, IN.texCoord.xy).x;
	float4 tempColor = float4(tex2D(g_samplerSourceMap, IN.texCoord.xy).xyz, 1);
	
	float multiplier = 1.0f;
	multiplier = 0.5f;
	
	OUT.diffuse = tempColor * (1 - 0.7f*nbShadows*invMaxShadowMaps);
		
	return OUT;
}

PS_OUTPUT AdvancedCombinePS(vertexOutputCP IN) {
	PS_OUTPUT OUT = (PS_OUTPUT) 0;
	
	float4 retColor = tex2D(g_samplerShadowMap, IN.texCoord.xy);
	OUT.diffuse = retColor*invMaxShadowMaps;
	
	return OUT;
}

PS_OUTPUT sumAfterwardsPS(vertexOutputCP IN) {
	PS_OUTPUT OUT = (PS_OUTPUT) 0;
	
	float newShadow = tex2D(g_samplerShadowMap, IN.texCoord.xy).x;
	float alreadySummed = tex2D(g_samplerSourceMap, IN.texCoord.xy).x;
	
	OUT.diffuse.x = alreadySummed.x + newShadow.x;
	
	return OUT;
}


// 
// Effect
//

technique ShadowMap
{
    pass Pass0
    {
        VertexShader = compile vs_2_0 ShadowMapVertexShader();
        PixelShader  = compile ps_2_0 ShadowMapPixelShader();
        CullMode         = CCW;
        Lighting		 = false;
    }
}

technique ShowDepthMaps
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 ShowDepthMapsVS();
        PixelShader  = compile ps_3_0 ShowDepthMapsPS();
        CullMode         = CCW;
        Lighting		 = false;
    }
}

technique Copy
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 CopyVS();
        PixelShader  = compile ps_3_0 CopyPS();
        CullMode         = CCW;
        Lighting		 = false;
    }
}

technique Reprojection
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 ReprojectionVS();
        PixelShader  = compile ps_3_0 ReprojectionPS();
        CullMode         = CCW;
        Lighting		 = false;
    }
}

technique CompareDepthMaps
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 CompareDepthMapsVS();
        PixelShader  = compile ps_3_0 CompareDepthMapsPS();
        CullMode         = CCW;
        Lighting		 = false;
    }
}

technique ClearDepthMap
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 ClearDepthMapVS();
        PixelShader  = compile ps_3_0 ClearDepthMapPS();
        CullMode         = CCW;
        Lighting		 = false;
    }
}

technique Combine
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 CopyVS();
        PixelShader  = compile ps_3_0 CombinePS();
        CullMode         = CCW;
        Lighting		 = false;
    }
}

technique AdvancedCombine
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 CopyVS();
        PixelShader  = compile ps_3_0 AdvancedCombinePS();
        CullMode         = CCW;
        Lighting		 = false;
    }
}

technique SumAfterwards
{
    pass Pass0
    {
        VertexShader = compile vs_3_0 CopyVS();
        PixelShader  = compile ps_3_0 sumAfterwardsPS();
        CullMode         = CCW;
        Lighting		 = false;
    }
}