//------------------------------------
float alpha;

texture objTexture;
texture ShadowMap;
texture renderTarget;
texture sceneDepth;

float4x4 g_mWorldViewProjection;
float4x4 g_mWorldViewIT;
float4x4 g_mLightWorldViewProjection;

float invHalfScreenWidth;
float invHalfScreenHeight;

float4 lightDir;
float3 halfVector;
float4 ambient;
float4 diffuse;
float4 specular;
float  power;
float FarPlaneMinusNearPlane;
float SMBias;

float samplingDistance;

int kernelSize;

//------------------------------------
struct vertexObjInput {
    float3 position		: POSITION;
    float4 texCoord	: TEXCOORD0;
    float3 normal		: NORMAL;
};

struct vertexObjOutput {
    float4 HPosition			: POSITION;
    float3 normal				: NORMAL;
    
    float4 ColorTextureCoord	: TEXCOORD7;
};

struct vertexObjSumOutput {
    float4 HPosition			: POSITION;
    float4 ShadowMapSamplingPos0: TEXCOORD0;
    float4 Pos2d				: TEXCOORD1;
};

struct vertexObjASumOutput {
    float4 HPosition			: POSITION;
    float3 normal				: NORMAL;
    float4 ShadowMapSamplingPos0: TEXCOORD0;
    float4 Pos2d				: TEXCOORD1;
    float4 ColorTextureCoord	: TEXCOORD7;
};

struct pixelOutput
{
	float4 color		: COLOR;
};


//------------------------------------
sampler samplerTexture = sampler_state
{
	Texture   = (objTexture);
    MinFilter = LINEAR; 
    MagFilter = LINEAR; 
    MipFilter = LINEAR;
    AddressU  = BORDER;
    AddressV  = BORDER;
};

sampler samplerShadowMap = sampler_state 
{ 
  texture = <ShadowMap>; 
  magfilter = LINEAR; 
  minfilter = LINEAR; 
  mipfilter = LINEAR; 
  AddressU = mirror; 
  AddressV = mirror;
};

sampler samplerRenderTexture = sampler_state 
{ 
  texture = <renderTarget>; 
  magfilter = LINEAR; 
  minfilter = LINEAR; 
  mipfilter = LINEAR; 
  AddressU = mirror; 
  AddressV = mirror;
};

sampler samplerSceneDepthMap = sampler_state 
{ 
  texture = <sceneDepth>; 
  magfilter = LINEAR; 
  minfilter = LINEAR; 
  mipfilter = LINEAR; 
  AddressU = mirror; 
  AddressV = mirror;
};

//------------------------------------
vertexObjOutput simpleMeshVS(vertexObjInput IN) 
{
	vertexObjOutput OUT = (vertexObjOutput)0;
    
	OUT.HPosition = mul(float4(IN.position,1), g_mWorldViewProjection);
	OUT.normal = normalize(mul(g_mWorldViewIT, float4(IN.normal,1))).xyz;
	OUT.ColorTextureCoord = IN.texCoord;
    
	return OUT;
}

vertexObjSumOutput sumShadowsVS(vertexObjInput IN) 
{
	vertexObjSumOutput OUT = (vertexObjSumOutput)0;
    
	OUT.HPosition = mul(float4(IN.position,1), g_mWorldViewProjection);
	
	OUT.Pos2d = OUT.HPosition;
	OUT.Pos2d.z /= (FarPlaneMinusNearPlane * 0.5);
	
	OUT.ShadowMapSamplingPos0 = mul(float4(IN.position,1), g_mLightWorldViewProjection);
	
	//saving real distance in z:
	OUT.ShadowMapSamplingPos0.z = OUT.ShadowMapSamplingPos0.z / (FarPlaneMinusNearPlane * 0.5);
	
	return OUT;
}


//------------------------------------
pixelOutput simpleMeshPS(vertexObjOutput IN)
{
	pixelOutput OUT;
	float3 normal = IN.normal;
	float4 retColor = tex2D(samplerTexture, IN.ColorTextureCoord.xy);
	
	//Specular
	float3 specI = specular*pow( clamp(0, 1, dot(normal, halfVector)), power);
	
	//Diffuse
	float3 diffI = diffuse*clamp(0,1,dot(normal, lightDir));
	
	//Ambient = ambient
	retColor.rgb = retColor.rgb*(diffI + specI + ambient.xyz);
	retColor.a *= alpha;
	
	//Do Lighting Stuff
	OUT.color = retColor;
	
	return OUT;
}

pixelOutput simpleMeshNTPS(vertexObjOutput IN)
{
	pixelOutput OUT;
	float3 normal = IN.normal;
	float4 retColor = float4(1, 1, 1, 0);//tex2D(samplerTexture, IN.ColorTextureCoord.xy);
	
	//Specular
	float3 specI = specular*pow( clamp(0, 1, dot(normal, halfVector)), power);
	
	//Diffuse
	float3 diffI = diffuse*clamp(0,1,dot(normal, lightDir));
	
	//Ambient = ambient
	retColor.rgb = retColor.rgb*(diffI + specI + ambient.xyz);
	retColor.a *= alpha;
	
	//Do Lighting Stuff
	OUT.color = retColor;
	
	return OUT;
}

pixelOutput sumShadowsPS(vertexObjSumOutput IN)
{
	pixelOutput OUT = (pixelOutput)0;
	
	// do the lookup in the render target (get current shadow sum):
	float2 lookupCoord;
	lookupCoord[0] = IN.Pos2d.x/IN.Pos2d.w/2.0f +0.5f + invHalfScreenWidth;
	lookupCoord[1] = -IN.Pos2d.y/IN.Pos2d.w/2.0f +0.5f + invHalfScreenHeight;
	OUT.color.xyzw = tex2D(samplerRenderTexture, lookupCoord);
	
	float2 samplingCoord;
	samplingCoord[0] = IN.ShadowMapSamplingPos0.x/IN.ShadowMapSamplingPos0.w/2.0f +0.5f;
	samplingCoord[1] = -IN.ShadowMapSamplingPos0.y/IN.ShadowMapSamplingPos0.w/2.0f +0.5f;
		
	if ((saturate(samplingCoord.x) == samplingCoord.x) && (saturate(samplingCoord.y) == samplingCoord.y))
	{ 
		float StoredDepthInShadowMap = tex2D(samplerShadowMap, samplingCoord).x;
		//real distance is stored in .z!
		if ((IN.ShadowMapSamplingPos0.z - SMBias) > StoredDepthInShadowMap)
		{
			//this one is in shadow - but we have to check if there is another object in front of it!
			//==>do the depth lookup to find out!
			float realDepth  = tex2D(samplerSceneDepthMap, lookupCoord).x;
			
			if(abs(IN.Pos2d.z - realDepth) < 0.002f) {
			//if(IN.Pos2d.z == realDepth) {
				OUT.color.r += 1.0f;
			}
			
		}	
	} else {
		OUT.color.r += 1.0f;
	}
	
	//Write Detph into green channel
	OUT.color.g = IN.Pos2d.z;
	
	return OUT;
}

vertexObjASumOutput sumAdvancedShadowsVS(vertexObjInput IN) 
{
	vertexObjASumOutput OUT = (vertexObjASumOutput)0;
    
	OUT.HPosition = mul(float4(IN.position,1), g_mWorldViewProjection);
	
	OUT.Pos2d = OUT.HPosition;
	OUT.Pos2d.z /= (FarPlaneMinusNearPlane * 0.5);
	
	
	OUT.ShadowMapSamplingPos0 = mul(float4(IN.position,1), g_mLightWorldViewProjection);
	
	//saving real distance in z:
	OUT.ShadowMapSamplingPos0.z = OUT.ShadowMapSamplingPos0.z / (FarPlaneMinusNearPlane * 0.5);
	
	OUT.normal = normalize(mul(g_mWorldViewIT, float4(IN.normal,1))).xyz;
	OUT.ColorTextureCoord = IN.texCoord;
	
	return OUT;
}

pixelOutput sumAdvancedShadowsPS(vertexObjASumOutput IN)
{
	pixelOutput OUT;
	
	// do the lookup in the render target (get current shadow sum):
	float2 lookupCoord;
	lookupCoord[0] = IN.Pos2d.x/IN.Pos2d.w*0.5f +0.5f + invHalfScreenWidth;
	lookupCoord[1] = -IN.Pos2d.y/IN.Pos2d.w*0.5f +0.5f + invHalfScreenHeight;
	
	OUT.color.xyzw = tex2D(samplerRenderTexture, lookupCoord);
	
	float2 samplingCoord;
	samplingCoord[0] = IN.ShadowMapSamplingPos0.x/IN.ShadowMapSamplingPos0.w/2.0f +0.5f;
	samplingCoord[1] = -IN.ShadowMapSamplingPos0.y/IN.ShadowMapSamplingPos0.w/2.0f +0.5f;
		
	float StoredDepthInShadowMap = tex2D(samplerShadowMap, samplingCoord).x;
	
	
	float4 retColor = float4(0,0,0,0);
	
	//real distance is stored in .z!
		
	float3 normal = IN.normal;
	retColor = tex2D(samplerTexture, IN.ColorTextureCoord.xy);
	
	//Specular
	float3 specI = specular*pow( clamp(0, 1, dot(normal, halfVector)), power);
	
	//Diffuse
	float3 diffI = diffuse*clamp(0,1,dot(normal, lightDir));
	
	//Ambient = ambient
	retColor.rgb = retColor.rgb*(diffI + specI + ambient.xyz);
	retColor.a *= alpha;
		
	if ((IN.ShadowMapSamplingPos0.z - SMBias) > StoredDepthInShadowMap &&
		(saturate(samplingCoord.x) == samplingCoord.x) && 
		(saturate(samplingCoord.y) == samplingCoord.y))
	{	
		retColor *= 0.5f;	
	}
	
	//Do Lighting Stuff
	OUT.color += retColor;
	
	return OUT;
}

pixelOutput singleShadowOnlyPS(vertexObjSumOutput IN)
{
	pixelOutput OUT = (pixelOutput)0;
	
	OUT.color.xyzw = 0.0f;
	
	float2 samplingCoord;
	samplingCoord[0] = IN.ShadowMapSamplingPos0.x/IN.ShadowMapSamplingPos0.w/2.0f +0.5f;
	samplingCoord[1] = -IN.ShadowMapSamplingPos0.y/IN.ShadowMapSamplingPos0.w/2.0f +0.5f;
		
	if ((saturate(samplingCoord.x) == samplingCoord.x) && (saturate(samplingCoord.y) == samplingCoord.y))
	{ 
		float StoredDepthInShadowMap = tex2D(samplerShadowMap, samplingCoord).x;
		//real distance is stored in .z!
		if ((IN.ShadowMapSamplingPos0.z - SMBias) > StoredDepthInShadowMap)
		{
			OUT.color.r = 1.0f;			
		}	
	} else {
		OUT.color.r = 1.0f;
	}
	
	return OUT;
}

pixelOutput sumShadowsPCFPS(vertexObjSumOutput IN)
{
	pixelOutput OUT = (pixelOutput)0;
	
	// do the lookup in the render target (get current shadow sum):
	float2 lookupCoord;
	lookupCoord[0] = IN.Pos2d.x/IN.Pos2d.w/2.0f +0.5f + invHalfScreenWidth;
	lookupCoord[1] = -IN.Pos2d.y/IN.Pos2d.w/2.0f +0.5f + invHalfScreenHeight;
	OUT.color.xyzw = tex2D(samplerRenderTexture, lookupCoord);
	
	float2 samplingCoord;
	samplingCoord[0] = (IN.ShadowMapSamplingPos0.x/IN.ShadowMapSamplingPos0.w/2.0f +0.5f) - ((int)(kernelSize / 2.0f)) * samplingDistance;
	samplingCoord[1] = (-IN.ShadowMapSamplingPos0.y/IN.ShadowMapSamplingPos0.w/2.0f +0.5f) - ((int)(kernelSize / 2.0f)) * samplingDistance;
	float PCFSum = 0.0f;
	float2 currentSamplingCoord;
		
	for(int i = 0; i < kernelSize; i++) {
		for(int j = 0; j < kernelSize; j++) {
			currentSamplingCoord[0] = samplingCoord.x + i*samplingDistance;
			currentSamplingCoord[1] = samplingCoord.y + j*samplingDistance;
			
			if ((saturate(currentSamplingCoord.x) == currentSamplingCoord.x) && (saturate(currentSamplingCoord.y) == currentSamplingCoord.y))
			{ 
				float StoredDepthInShadowMap = tex2D(samplerShadowMap,currentSamplingCoord).x;
				//real distance is stored in .z!
				if ((IN.ShadowMapSamplingPos0.z - SMBias) > StoredDepthInShadowMap)
				{
					//this one is in shadow - but we have to check if there is another object in front of it!
					//==>do the depth lookup to find out!
					float realDepth  = tex2D(samplerSceneDepthMap, lookupCoord).x;
					
					if(abs(IN.Pos2d.z - realDepth) < 0.002f) {
						PCFSum += 1.0f;
					}
					
				}	
			} else {
				PCFSum += 1.0f;
			}
		}
	}
	
	PCFSum = PCFSum/(kernelSize * kernelSize);
	
	//Add PCF value to Sum Map:
	OUT.color.r += PCFSum;
	
	//Write Depth into green channel
	OUT.color.g = IN.Pos2d.z;
	
	return OUT;
}


pixelOutput sumShadowsPCF1DPS(vertexObjSumOutput IN)
{
	pixelOutput OUT = (pixelOutput)0;
	
	// do the lookup in the render target (get current shadow sum):
	float2 lookupCoord;
	lookupCoord[0] = IN.Pos2d.x/IN.Pos2d.w/2.0f +0.5f + invHalfScreenWidth;
	lookupCoord[1] = -IN.Pos2d.y/IN.Pos2d.w/2.0f +0.5f + invHalfScreenHeight;
	OUT.color.xyzw = tex2D(samplerRenderTexture, lookupCoord);
	
	float samplingCoord;
	samplingCoord = (IN.ShadowMapSamplingPos0.x/IN.ShadowMapSamplingPos0.w/2.0f +0.5f) - ((int)(kernelSize / 2.0f)) * samplingDistance;
	float PCFSum = 0.0f;
	
	float2 currentSamplingCoord;
	currentSamplingCoord[1] = (-IN.ShadowMapSamplingPos0.y/IN.ShadowMapSamplingPos0.w/2.0f +0.5f);
		
	for(int i = 0; i < kernelSize; i++) {
		currentSamplingCoord[0] = samplingCoord + i*samplingDistance;
			
		if ((saturate(currentSamplingCoord.x) == currentSamplingCoord.x) && (saturate(currentSamplingCoord.y) == currentSamplingCoord.y))
		{ 
			float StoredDepthInShadowMap = tex2D(samplerShadowMap,currentSamplingCoord).x;
			//real distance is stored in .z!
			if ((IN.ShadowMapSamplingPos0.z - SMBias) > StoredDepthInShadowMap)
			{
				//this one is in shadow - but we have to check if there is another object in front of it!
				//==>do the depth lookup to find out!
				float realDepth  = tex2D(samplerSceneDepthMap, lookupCoord).x;
				
				if(abs(IN.Pos2d.z - realDepth) < 0.002f) {
					PCFSum += 1.0f;
				}
				
			}	
		} else {
			PCFSum += 1.0f;
		}
	}
	
	PCFSum = PCFSum/(kernelSize);
	
	//Add PCF value to Sum Map:
	OUT.color.r += PCFSum;
	
	//Write Depth into green channel
	OUT.color.g = IN.Pos2d.z;
	
	return OUT;
}


// 
// Effect
//


technique simpleMesh
{
    pass p0
    {
	    VertexShader = compile vs_3_0 simpleMeshVS();
	    PixelShader  = compile ps_3_0 simpleMeshPS();
	}
}

technique simpleMeshNT
{
    pass p0
    {
	    VertexShader = compile vs_3_0 simpleMeshVS();
	    PixelShader  = compile ps_3_0 simpleMeshNTPS();
	}
}

technique sumShadows
{
	pass P0
	{
		Vertexshader = compile vs_3_0 sumShadowsVS();
        Pixelshader  = compile ps_3_0 sumShadowsPS();
	}
}

technique sumAdvancedShadows
{
	pass P0
	{
		Vertexshader = compile vs_3_0 sumAdvancedShadowsVS();
        Pixelshader  = compile ps_3_0 sumAdvancedShadowsPS();
	}
}

technique singleShadowOnly
{
	pass P0
	{
		Vertexshader = compile vs_3_0 sumShadowsVS();
        Pixelshader  = compile ps_3_0 singleShadowOnlyPS();
	}
}

technique sumShadowsPCF
{
	pass P0
	{
		Vertexshader = compile vs_3_0 sumShadowsVS();
        Pixelshader  = compile ps_3_0 sumShadowsPCFPS();
	}
}

technique sumShadowsPCF1D
{
	pass P0
	{
		Vertexshader = compile vs_3_0 sumShadowsVS();
        Pixelshader  = compile ps_3_0 sumShadowsPCF1DPS();
	}
}