#pragma once

#include<vector> // Standard-Vektor einschlieen
#include "SoftShadowScene.h"
#include <string>
#include "ResourceManager.h"


using namespace std;

class Scene;
class Node;

#define SPTR boost::shared_ptr

/**
GameManager class
*/

class GameManager
{
public:
	/** 
	Constructor for a GameManager Object
	*/
	GameManager(void);
	/**
	Destructor for a GameManager Object
	*/
	~GameManager(void);

	/**
	Initializes the GameManager
	*/
	int init(void);
	
	/**
	Debug Output to Console
	*/
	void printToConsole (std::string output);
	/**
	Adds a scene to the scene vector
	*/
	void addScene(Scene &newScene);
	/**
	Removes a scene from the scene vector
	*/
	void removeScene (Scene &oldScene);
	/**
	Sets the active scene
	*/
	void setActiveScene(Scene &aScene);
	/**
	Performs an action whenever a key is pressed
	*/
	void keyPressed(UINT nChar, bool bKeyDown, bool bAltDown, void* pUserContext);
	/**
	Sets the current mouse status
	*/
	void setMouseStatus(bool bLeftButtonDown,  bool bRightButtonDown,bool bMiddleButtonDown,
                     bool bSideButton1Down, bool bSideButton2Down, int nMouseWheelDelta,
                     int xPos, int yPos);
	/**
	update the game/the program
	*/
	void updateGame(float fElapsedTime);
	/**
	initializes the game/the program
	*/
	void initGame();
	
	/**
	Returns the requested effect
	*/
	ID3DXEffect* getEffect(UINT id);
	/**
	Load an effect from a file
	*/
	ID3DXEffect* loadEffect(UINT id, LPCWSTR filename);
	/**
	Release an effect
	*/
	void releaseEffect(ID3DXEffect* effect);

	void OnLostDevice( void* pUserContext );
	void OnDestroyDevice( void* pUserContext );
	HRESULT OnCreateDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext );
	HRESULT OnResetDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext );
	void OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext );

	/**
	Sets screen Dimensions
	*/
	void setScreenDimension(int width, int height);

	/**
	Resource Manager object
	*/
	ResourceManager resManager;

	int screenWidth;
	int screenHeight;

	int screenResolution[2];

	/**
	The SoftShadow scene
	*/
	SoftShadowScene gs;  // Softshadow Scene

	//EffectConstants
	static const UINT EFFECT_FINAL		= 0;
	static const UINT EFFECT_SIMPLEMESH	= 3;
	static const UINT EFFECT_SHADOWMAP	= 6;
	static const UINT EFFECT_DEPTHMAP	= 7;
	
	friend class Scene;

protected:
	Scene* activeScene;
	std::vector<Scene *> sceneVector;
	bool consoleStarted;
	HANDLE outputHandle;

	//Effect vector
	static const int NB_EFFECTS = 100;
	ID3DXEffect* effectList[NB_EFFECTS];
	int effectReleaseCount[NB_EFFECTS];

	bool firstFrame;
	IDirect3DSurface9* finalRenderTarget;
	IDirect3DDevice9* device;
	ID3DXEffect* finalEffect;
	ID3DXEffect* shadowMapEffect;
	ID3DXEffect* depthMapEffect;
	ID3DXEffect* simpleMeshEffect;
};
