#include "dxstdafx.h"
#include "LowMemLight.h"
#include "Scene.h"
#include "GameManager.h"

LowMemLight::LowMemLight(void)
{	length = 5;
	epsilon = 1;
	mapSize = 512;
	lowResMapSize = 512;

	errorMetric = LowMemLight::ERR_METRIC_PIXCOUNT;
	this->errorMap = NULL;
	this->errorSurface = NULL;
	this->errorThreshold = 0.0125f;

	currentMap = NULL;
	currentWorldMatrix = NULL;
	errorMap = NULL;
	errorSurface = NULL;

	firstLightSumTexture = NULL;
	firstLightSumSurface = NULL;
	secondLightSumTexture = NULL;
	secondLightSumSurface = NULL;
	thirdLightSumTexture = NULL;
	thirdLightSumSurface = NULL;

	this->setSMBias(1.0f/2000.0f);
	this->setFilterKernelSize(5);
	this->setFilterSamplingDistance(0.008f);
}

LowMemLight::~LowMemLight(void)
{
	if(errorSurface) SAFE_RELEASE(errorSurface);
	if(errorMap) SAFE_RELEASE(errorMap);
	if(firstLightSumTexture) SAFE_RELEASE(firstLightSumTexture);
	if(firstLightSumSurface) SAFE_RELEASE(firstLightSumSurface);
	if(secondLightSumTexture) SAFE_RELEASE(secondLightSumTexture);
	if(secondLightSumSurface) SAFE_RELEASE(secondLightSumSurface);
	if(thirdLightSumTexture) SAFE_RELEASE(thirdLightSumTexture);
	if(thirdLightSumSurface) SAFE_RELEASE(thirdLightSumSurface);
}

void LowMemLight::setLightDimension(float _length)
{
	length = _length;
}

void LowMemLight::setEpsilon(float _epsilon)
{
	epsilon = _epsilon;
}

void LowMemLight::setErrorThreshold(float _errorThreshold)
{
	errorThreshold = _errorThreshold;
}

float LowMemLight::getErrorThreshold() {
	return this->errorThreshold;
}

void LowMemLight::setShadowMapSize(int _mapSize)
{
	HRESULT hr;
	mapSize = _mapSize;
	if(firstLightSumTexture) SAFE_RELEASE(firstLightSumTexture);
	if(firstLightSumSurface) SAFE_RELEASE(firstLightSumSurface);
	if(secondLightSumTexture) SAFE_RELEASE(secondLightSumTexture);
	if(secondLightSumSurface) SAFE_RELEASE(secondLightSumSurface);
	if(thirdLightSumTexture) SAFE_RELEASE(thirdLightSumTexture);
	if(thirdLightSumSurface) SAFE_RELEASE(thirdLightSumSurface);


	if(currentMap) SAFE_RELEASE(currentMap);

	V( DXUTGetD3DDevice()->CreateTexture( myScene->manager->screenWidth, myScene->manager->screenHeight, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A32B32G32R32F, D3DPOOL_DEFAULT, &firstLightSumTexture, NULL ) );
	V( firstLightSumTexture->GetSurfaceLevel(0, &firstLightSumSurface) );
	V( DXUTGetD3DDevice()->CreateTexture( myScene->manager->screenWidth, myScene->manager->screenHeight, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A32B32G32R32F, D3DPOOL_DEFAULT, &secondLightSumTexture, NULL ) );
	V( secondLightSumTexture->GetSurfaceLevel(0, &secondLightSumSurface) );
	if(this->myScene->sumMode == this->myScene->SUM_TWOPASS) {
		V( DXUTGetD3DDevice()->CreateTexture( myScene->manager->screenWidth, myScene->manager->screenHeight, 1, D3DUSAGE_RENDERTARGET, D3DFMT_A32B32G32R32F, D3DPOOL_DEFAULT, &thirdLightSumTexture, NULL ) );
		V( thirdLightSumTexture->GetSurfaceLevel(0, &thirdLightSumSurface) );
	}

	//Use second and first so that rendertarget and sourcetexture are different maps
	cLightSumTexture = firstLightSumTexture;
	cLightSumSurface = secondLightSumSurface;
}

void LowMemLight::setLowResShadowMap(int _lowResMapSize)
{
	HRESULT hr;
	lowResMapSize = _lowResMapSize;

	if(errorMap) SAFE_RELEASE(errorMap);
	if(errorSurface) SAFE_RELEASE(errorSurface);

	V( DXUTGetD3DDevice()->CreateTexture( lowResMapSize, lowResMapSize, 1, D3DUSAGE_RENDERTARGET, D3DFMT_R32F, D3DPOOL_DEFAULT, &errorMap, NULL ) );
	V( errorMap->GetSurfaceLevel(0, &errorSurface) );
}

float LowMemLight::getLightDimension()
{
	return length;
}

float LowMemLight::getEpsilon()
{
	return epsilon;
}

int LowMemLight::getShadowMapSize()
{
	return mapSize;
}

void LowMemLight::getAbsoluteVector(Vector &pOut, Vector &pIn) 
{
	D3DXMATRIXA16 invViewMatrix;
	D3DXMatrixInverse(&invViewMatrix, NULL, &this->myWorldMatrix);
	D3DXVec4Transform(&pOut, &pIn, &invViewMatrix);
}

void LowMemLight::setNbShadowMaps(int _nbMaps)
{
	nbMaps = _nbMaps;
}

int LowMemLight::getNbShadowMaps()
{
	return nbMaps;
}

IDirect3DTexture9* LowMemLight::getCurrentDepthMap()
{
	return currentMap;
}

D3DXMATRIX* LowMemLight::getCurrentWorldMatrix()
{
	return currentWorldMatrix;
}

void LowMemLight::setCurrentDepthMap(IDirect3DTexture9* _currentDepthMap)
{
	currentMap = _currentDepthMap;
}

void LowMemLight::setCurrentWorldMatrix(D3DXMATRIX* _currentWorldMatrix)
{
	currentWorldMatrix = _currentWorldMatrix;
}

IDirect3DSurface9* LowMemLight::getLightSumRenderTarget()
{
	return cLightSumSurface;
}

IDirect3DSurface9* LowMemLight::getThirdRenderTarget()
{
	return thirdLightSumSurface;
}

IDirect3DSurface9* LowMemLight::getErrorSurface()
{
	return this->errorSurface;
}

IDirect3DTexture9* LowMemLight::getErrorTexture()
{
	return this->errorMap;
}

int LowMemLight::getLowResShadowMapSize()
{
	return this->lowResMapSize;
}


IDirect3DTexture9* LowMemLight::getLightSumTexture()
{
	return this->cLightSumTexture;
}

IDirect3DTexture9* LowMemLight::getThirdTexture()
{
	return this->thirdLightSumTexture;
}

//************** ERROR METRIC CALCULATION **************

void LowMemLight::setErrorMetric(int _errorMetric) 
{
	errorMetric = _errorMetric;
}

bool LowMemLight::isErrorOkay(DWORD nbPixels)
{
	HRESULT hr;

	if(errorMetric == LowMemLight::ERR_METRIC_PIXCOUNT) {
		//Standardmetric
		float tempPixels = ((float) nbPixels)/((1.0f*mapSize)*mapSize);
		return epsilon>=tempPixels;
	} else if(errorMetric == LowMemLight::ERR_METRIC_MAXAREACOUNT) {
		int x = 0, y = 0;
		int maxArea = 0;
		int fieldCounter = 2;

		//Lock Texture
		D3DLOCKED_RECT lockedRect;
		V( errorSurface->LockRect(&lockedRect, NULL, 0) );
		float* floatPointer = (float*) lockedRect.pBits;
		int pitch = lockedRect.Pitch;

		//While nextArea found
		while(findNextArea(x,y, floatPointer, pitch)) {
			//Count pixels
			countAndLabelPixels(x, y, floatPointer, maxArea, fieldCounter, pitch);
		}

		//Unlock Texture
		V( errorSurface->UnlockRect() );
	}
	myScene->manager->printToConsole("ERROR: WRONG ERROR METRIC DEFINED!");
	return false;
}

bool LowMemLight::findNextArea(int &x, int &y, float* floatPointer, int pitch) {
	float value;
	char temp[100];
	for(int i=0;i<this->mapSize;i++) {
		for(int j=0;j<this->mapSize;j++) {
			value = floatPointer[i*pitch + j];
			sprintf_s(temp, "value = %f", value);
			myScene->manager->printToConsole(temp);
		}
	}
	return false;
}

void LowMemLight::countAndLabelPixels(int &x, int &y, float* floatPointer, int &maxArea, int &fieldCounter, int pitch) {
}

void LowMemLight::swapLightSumMaps()
{
	if(this->cLightSumTexture == this->firstLightSumTexture) {
		this->cLightSumTexture = this->secondLightSumTexture;
		this->cLightSumSurface = this->firstLightSumSurface;
	} else {
		this->cLightSumTexture = this->firstLightSumTexture;
		this->cLightSumSurface = this->secondLightSumSurface;
	}
}

void LowMemLight::setSMBias(float _fSMBias) {
	this->fSMBias = _fSMBias;
}

void LowMemLight::setFilterKernelSize(int _kernelSize) {
	this->iKernelSize = _kernelSize;
}

void LowMemLight::setFilterSamplingDistance(float _samplingDistance) {
	this->fSamplingDistance = _samplingDistance;
}

int LowMemLight::getFilterKernelSize() {
	return this->iKernelSize;
}

float LowMemLight::getFilterSamplingDistance() {
	return this->fSamplingDistance;
}

float LowMemLight::getSMBias() {
	return this->fSMBias;
}

void LowMemLight::calcWorldMatrix(D3DXMATRIX &pMatWorld)
{
	D3DXQUATERNION quat;
	D3DXMATRIX tempWorld;
	D3DXMATRIX tempTrans;
    D3DXMATRIX MatRot;   // Final rotation matrix, applied to 
                         // pMatWorld.

	float x = -this->myPosition.x;
	float y = -this->myPosition.y;
	float z = -this->myPosition.z;

	float a = this->myRotation.x;
	float b = this->myRotation.y;
	float g = this->myRotation.z;

	D3DXMatrixTranslation(&tempTrans, x, y, z);
	if(a!=0 || b!=0 || g!=0) {
		D3DXQuaternionIdentity(&quat);
		D3DXQuaternionRotationYawPitchRoll(&quat, b, a, g);
		D3DXMatrixRotationQuaternion(&MatRot, &quat);
		D3DXMatrixMultiply(&this->myWorldMatrix, &MatRot, &pMatWorld);
	}
	D3DXMatrixMultiply(&this->myWorldMatrix, &tempTrans, &this->myWorldMatrix);
}