#pragma once
#include "node.h"

/**
LowMemLight class
*/

class LowMemLight :
	public Node
{
public:
	/**
	Constructor for a softshadow light
	*/
	LowMemLight(void);
	/**
	Destructor for a softshadow light
	*/
	~LowMemLight(void);

	/**
	Sets the dimension of the light
	*/
	void setLightDimension(float _length);
	/**
	Sets the upper errorbound
	*/
	void setEpsilon(float _epsilon);
	/**
	This factor defines how big the difference between to pixels may be,
	so that they are treated as different ones
	*/
	void setErrorThreshold(float _errorThreshold);
	/**
	Sets the filter kernel size
	*/
	void setFilterKernelSize(int _kernelSize);
	/**
	Sets the sampling distance for the filter kernel
	*/
	void setFilterSamplingDistance(float _samplingDistance);
	/**
	Sets the shadowmap bias
	*/
	void setSMBias(float _fSMBias);
	/**
	Sets the shadowmap size
	*/
	void setShadowMapSize(int _mapSize);
	/**
	Sets the lowResShadowMap, currently its always the same size as normal shadowMap
	*/
	void setLowResShadowMap(int _lowResMapSize);
	/**
	Sets the errorMetric. Only defualt metric supported
	*/
	void setErrorMetric(int _errorMetric);
	/**
	Sets the number of shadowmaps
	*/
	void setNbShadowMaps(int _nbMaps);
	/**
	Sets the current depthMap
	*/
	void setCurrentDepthMap(IDirect3DTexture9* _currentDepthMap);
	/**
	Sets the current worldMatrix
	*/
	void setCurrentWorldMatrix(D3DXMATRIX* _currentWorldMatrix);
	
	/**
	Returns lightDimension
	*/
	float getLightDimension();
	/**
	Returns upper bound epsilon
	*/
	float getEpsilon();
	/**
	Returns the error Threshold
	*/
	float getErrorThreshold();
	/**
	Returns shadowmap bias
	*/
	float getSMBias();
	/**
	Returns the filter kernel size
	*/
	int getFilterKernelSize();
	/**
	Returns the sampling distance for the filter kernel
	*/
	float getFilterSamplingDistance();
	/**
	Returns the shadowmap size
	*/
	int getShadowMapSize();
	/**
	Returns the lowres shadowmap size
	*/
	int getLowResShadowMapSize();
	/**
	Returns the number of shadowmaps
	*/
	int getNbShadowMaps();
	/**
	Returns the current depthmap
	*/
	IDirect3DTexture9* getCurrentDepthMap();
	/**
	Returns the current world matrix
	*/
	D3DXMATRIX* getCurrentWorldMatrix();
	/**
	Returns the current lightsum rendertarget
	*/
	IDirect3DSurface9* getLightSumRenderTarget();
	/**
	Returns the current lightsum texture
	*/
	IDirect3DTexture9* getLightSumTexture();
	/**
	Returns the absolute vector for a given inputvector
	*/
	virtual void getAbsoluteVector(Vector &pOut, Vector &pIn);
	/**
	Returns the error surface (indicator for the error )
	*/
	IDirect3DSurface9* getErrorSurface();
	/**
	Returns the error Texture
	*/
	IDirect3DTexture9* getErrorTexture();
	/**
	Returns a third rendertarget
	*/
	IDirect3DSurface9* getThirdRenderTarget();
	/**
	Returns the third sum texture
	*/
	IDirect3DTexture9* getThirdTexture();

	/**
	Swaps the light sum textures
	*/
	void swapLightSumMaps();

	/**
	Checks if a given error is lower between epsilon
	*/
	bool isErrorOkay(DWORD nbPixels);

	static const int ERR_METRIC_PIXCOUNT = 0;
	static const int ERR_METRIC_MAXAREACOUNT = 1;

	bool kernelSizeChanged;

	/**
	Calcs the World matrix
	*/
	virtual void calcWorldMatrix(D3DXMATRIX &pMatWorld);
protected:
	float length;
	float epsilon;
	int mapSize;
	int lowResMapSize;
	int errorMetric;
	float errorThreshold;
	int nbMaps;

	IDirect3DTexture9* currentMap;
	D3DXMATRIX* currentWorldMatrix;
	IDirect3DTexture9* errorMap;
	IDirect3DSurface9* errorSurface;
	IDirect3DTexture9* firstLightSumTexture;
	IDirect3DSurface9* firstLightSumSurface;
	IDirect3DTexture9* secondLightSumTexture;
	IDirect3DSurface9* secondLightSumSurface;
	IDirect3DTexture9* thirdLightSumTexture;
	IDirect3DSurface9* thirdLightSumSurface;
	IDirect3DTexture9* cLightSumTexture;
	IDirect3DSurface9* cLightSumSurface;
	
	//ErrorMetric functions
	/**
	Finds the next Area, for the labeling algorithm (not yet implemented)
	*/
	bool findNextArea(int &x, int &y, float* floatPointer, int pitch);
	/**
	Counts the number of pixels of each label (not yet implemented)
	*/
	void countAndLabelPixels(int &x, int &y, float* floatPointer, int &maxArea, int &fieldCounter, int pitch);

	float fSMBias;
	int iKernelSize;
	float fSamplingDistance;
};
