#include "dxstdafx.h"
#include "./Object3d.h"

#include "GameManager.h"
#include "Scene.h"
#include <stdio.h>

Object3d::Object3d(void) : Node()
{
	myMesh = NULL;
	
	adjacencyInfo = NULL;
	g_dwNumMaterials = 0;
	
	this->modelLoaded = false;
	this->isManagedByResourceManager = false;
	this->deleteMesh = true;

	this->maxFaces = 0;
	this->minFaces = 0;
	this->nodeType |= Scene::NODE_OBJECT3D;
	this->xfilename = "";

	iSceneNumber = 0;
	bFixed = false;
}

Object3d::~Object3d(void)
{
	if(myMesh!=NULL && !(isManagedByResourceManager)) {
		(myMesh)->Release();
	}
	
	if(!(isManagedByResourceManager)) {
		delete [] adjacencyInfo;
	}

	this->Materials.clear();
}

LPD3DXMESH* Object3d::getMesh()
{
	return &myMesh;
}

void Object3d::setMesh(LPD3DXMESH *_mesh) {
	this->myMesh = *_mesh;
}

void Object3d::loadMeshFromFile(std::string filename) 
{
	bool success = this->myScene->manager->resManager.loadXModel(filename, this);
	if(!success) {
		myMesh = NULL;
		this->g_dwNumMaterials = 0;
		this->modelLoaded = false;
		this->myScene->manager->printToConsole("loading model failed!");
	} else {
		this->xfilename = filename;
	}

}

void Object3d::computeAdjacency () {
	if(myMesh!=NULL) {
		if (adjacencyInfo != NULL)
			delete [] adjacencyInfo;
		
		//generate new DWORD array
		adjacencyInfo = new DWORD[(myMesh)->GetNumFaces() * 3];
		memset(adjacencyInfo,0,(myMesh)->GetNumFaces() * 3);

		// Get the adjacency info of the mesh.
		(myMesh)->GenerateAdjacency(0.0f, adjacencyInfo);
	}
}

void Object3d::computeNormals() 
{
	if(myMesh!=NULL) {
		// if mesh vertices have no normal variables defined, add them
		if ( !((myMesh)->GetFVF() & D3DFVF_NORMAL))
		{
			// define a temp mesh for the mesh with normal variables
			LPD3DXMESH pTempMesh;

			// clone the mesh, add normal variables
			(myMesh)->CloneMeshFVF((myMesh)->GetOptions(), 
									(myMesh)->GetFVF() | D3DFVF_NORMAL,
									DXUTGetD3DDevice(), 
									&pTempMesh 
									);
			
			// release the old mesh
			(myMesh)->Release();
			myMesh = pTempMesh;
		}

		if (adjacencyInfo == NULL)
			computeAdjacency();

		D3DXComputeNormals(
			myMesh,
			adjacencyInfo
		);
	}
}

bool Object3d::isModelLoaded()
{
	return this->modelLoaded;
}

void Object3d::setModelLoaded(bool value) {
	this->modelLoaded = value;
}

void Object3d::setXFilename(std::string _xfilename) {
	this->xfilename = _xfilename;
}

Node* Object3d::clone()
{
	Object3d* obj = (Object3d*) this->myScene->createNode(Scene::NODE_OBJECT3D,*this->father, true, false);
	this->cloneChildren(obj);
	return this->cloneObject3d(obj);
}

Object3d* Object3d::cloneObject3d(Object3d* obj)
{
	obj->setMesh(&this->myMesh);
	//set number of materials
	obj->g_dwNumMaterials = (DWORD)this->Materials.size();
	//set adjacency info
	obj->adjacencyInfo = this->adjacencyInfo;
	//set materials & textures:
	obj->Materials.resize(this->Materials.size());
	obj->Textures.resize(this->Textures.size());
	for (UINT i = 0; i < this->Materials.size(); i++) {
		obj->Materials[i] = this->Materials[i];
		obj->Textures[i] = this->Textures[i];
	}

	obj->minAABBox = this->minAABBox;
	obj->maxAABBox = this->maxAABBox;

	obj->setXFilename(this->xfilename);
	obj->isManagedByResourceManager = true;
	obj->setModelLoaded(true);
	obj->softKill = this->softKill;
	obj->softKillDuration = this->softKillDuration;

	obj->aClone = true;
	return obj;
}

void Object3d::OnDestroyDevice( void* pUserContext )
{
	if(myMesh!=NULL && !(isManagedByResourceManager)) {
		SAFE_RELEASE(myMesh);
	}
	
	if(!(isManagedByResourceManager)) {
		delete [] adjacencyInfo;
	}

	this->Textures.clear();
	this->Materials.clear();
}

HRESULT Object3d::OnResetDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
	if(this->xfilename.compare("")!=0) {
		this->loadMeshFromFile(this->xfilename);
	}
	return S_OK;
}