#pragma once
#include "./Node.h"
#include <string>
#include <boost/shared_ptr.hpp>
#include <vector>

using namespace std;

#define SPTR boost::shared_ptr

/**
Object3d class
*/

class Object3d : public Node {
public:
	/**
	Constructor for the Object3d object
	*/
	Object3d(void);
	/**
	Constructor for the Object3d object, with filename given
	*/
	Object3d(std::string file);
 	/**
	Destructor for the Object3d object
	*/
	~Object3d(void);

	/**
	Returns the object's mesh
	*/
	LPD3DXMESH* getMesh();
	/**
	Set the object's mesh
	*/
	void setMesh(LPD3DXMESH *mesh);
	/**
	Load a mesh from a given file
	*/
	void loadMeshFromFile(std::string filename);
	/**
	Compute the mesh normals
	*/
	void computeNormals();
	
	/**
	The object's mesh
	*/
	LPD3DXMESH myMesh;
		
	/**
	Returns whether a model has already been loaded
	*/
	bool isModelLoaded();
	/**
	Set the loaded status
	*/
	void setModelLoaded(bool value);
	/**
	Set the filename
	*/
	void setXFilename(std::string _xfilename);
	/**
	Adjacency info for the mesh
	*/
	DWORD *adjacencyInfo;
	
	/**
	Material vector
	*/
	std::vector<D3DMATERIAL9> Materials;
	/**
	Texture vector
	*/
	std::vector<IDirect3DTexture9*> Textures;
	/**
	number of materials used 
	*/
	DWORD g_dwNumMaterials;   // Number of materials

	/**
	Returns whether this object3d is managed by the resource manager
	*/
	bool isManagedByResourceManager;

	virtual Node* clone();
	
	virtual void OnDestroyDevice( void* pUserContext );
	virtual HRESULT OnResetDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext );

	/**
	The Scene number this object belongs to
	*/
	int iSceneNumber;
	/**
	Defines whether the object can be moved
	*/
	bool bFixed;
protected:
	Object3d* cloneObject3d(Object3d* obj);
	std::string xfilename;

	void computeAdjacency();

	bool modelLoaded;
	bool deleteMesh;

	int maxFaces;
	int minFaces;
	int numFacesPerStage[10];
	int currentFaces;
};