#pragma once
#include <vector>
#include <string>
#include "NodeFilter.h"
#include "Vector.h"

class Scene;
class Renderer;

//Gre des Arrays
#define SIZEOFCMD 3

/**
RenderPass class
*/

class RenderPass
{
public:
	/**
	Constructor for RenderPass object
	*/
	RenderPass(void);
	/**
	Destructor for RenderPass object
	*/
	~RenderPass(void);
	
	/**
	Set Renderer for this RenderPass
	*/
	void setRenderer(Renderer* _renderer);

	/**
	Make this a safe render target
	*/
	void setSafeAndRestoreRenderTarget(bool _safeRenderTarget);
	/**
	Set the LPDIRECT3DSURFACE9 render target
	*/
	void setRenderTarget(LPDIRECT3DSURFACE9 _renderTarget);
	/**
	Set depth stencil surface
	*/
	void setDepthStencilSurface(LPDIRECT3DSURFACE9 _depthStencilSurface);
	
	/**
	Set view matrix
	*/
	void setViewMatrix(D3DXMATRIX &_viewMat);
	/**
	Set projection matrix
	*/
	void setProjectionMatrix(D3DXMATRIX &_projMat);
	/**
	Set scene
	*/
	void setScene(Scene &_scene);

	static const int CMD_CLEARRENDERTARGETANDZBUFFER	= 0;
	static const int CMD_DOFRUSTUMCULLING				= 1;
	static const int CMD_RENDERNODES					= 2;
	
	/**
	Execute a command (0 = clear render target, 1 = frustum culling, 2 = render nodes)
	*/
	void execute(int command, bool _execute=true);
	/**
	Execute render pass
	*/
	virtual void executePass();

	/**
	Returns the view matrix
	*/
	D3DXMATRIX* getViewMatrix();
	/**
	Returns the projection matrix
	*/
	D3DXMATRIX* getProjectionMatrix();
	/**
	Returns the view projection matrix
	*/
	D3DXMATRIX* getViewProjectionMatrix();
	/**
	Returns the world view matrix
	*/
	D3DXMATRIX getWorldViewMatrix(D3DXMATRIX *worldMat);
	/**
	Returns the world light view matrix
	*/
	D3DXMATRIX getWorldLightViewMatrix(D3DXMATRIX *worldMat);
	/**
	Returns the world view projection matrix
	*/
	D3DXMATRIX getWorldViewProjectionMatrix(D3DXMATRIX *worldMat);

	/**
	Set RenderPass ID
	*/
	void setRenderPassId(int _passId);
	/**
	Returns the RenderPass ID
	*/
	int getRenderPassId();
	/**
	Set renderer list to use
	*/
	void useThisRendererList(std::vector<Renderer *>* _otherList);

	/**
	Calculate matrices
	*/
	void calculateMatrices();
	/**
	List of Renderers
	*/
	std::vector<Renderer *>* usedRendererList;
	/**
	The LPDIRECT3DSURFACE9 render target
	*/
	LPDIRECT3DSURFACE9 renderTarget;
protected:
	std::vector<Renderer *> myRendererList;
	Scene* scene;
	Renderer* renderer;
	IDirect3DDevice9* device;

	bool safeRenderTarget;
	LPDIRECT3DSURFACE9 renderTargetSurfaceOld;	
	LPDIRECT3DSURFACE9 depthStencilSurfaceOld;	
	LPDIRECT3DSURFACE9 dephStencilSurface;
	
	D3DXMATRIX viewMat;
	D3DXMATRIX projMat;
	D3DXMATRIX viewProjMat;

	bool executeCommand[SIZEOFCMD];
	int renderPassId;
};
