#pragma once
#include "Node.h"
#include <boost/shared_ptr.hpp>

class Scene;

#define SPTR boost::shared_ptr
#define WPTR boost::weak_ptr

/**
Renderer class
*/

class Renderer
{
public:
	friend class RenderPass;

	/**
	Constructor for Renderer object
	*/
	Renderer(void);
	/**
	Destructor for Renderer object
	*/
	virtual ~Renderer(void);
	/**
	Set the node to render
	*/
	void setNode(WPTR<Node> &_myNode);
	/**
	Set the scene this renderer belongs to
	*/
	void setScene(Scene &scene);

	/**
	Set render priority
	*/
	void setRenderPriority(int _priority);
	/**
	Returns render priority
	*/
	int getRenderPriority();

	/**
	Set the world matrix
	*/
	virtual void setWorldMatrix(D3DXMATRIX &_worldMat);
	/**
	Set the view matrix
	*/
	virtual void setViewMatrix(D3DXMATRIX &_viewMat);
	/**
	Set the projection matrix
	*/
	virtual void setProjectionMatrix(D3DXMATRIX &_projMat);

	/**
	Render the set node
	*/
	virtual void render() = 0;
	/**
	Init the renderer
	*/
	virtual void init() = 0;
	/**
	Returns whether the priority of this renderer is lower than the given one's
	*/
	virtual bool isLowerThan(Renderer* renderer) = 0;
	friend class Scene;

	virtual void OnLostDevice( void* pUserContext ) = 0;
	virtual void OnDestroyDevice( void* pUserContext ) = 0;
	virtual HRESULT OnResetDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext ) = 0;

	/**
	set the current texture to use during rendering
	*/
	virtual void setCurrentTexture(IDirect3DTexture9* texture);

	/**
	Returns whether this renderer is of given type
	*/
	bool isA(int _rendererType);
	static const int RENDERER_HUD = 1;
	static const int RENDERER_SIMPLEMESH = 16;
	static const int RENDERER_SKYBOX = 32;
	static const int RENDERER_SOFTSHADOWLIGHT = 128;

	/**
	Returns whether this renderer needs an extra pass
	*/
	virtual bool needsExtraPass();

	/**
	The set node
	*/
	WPTR<Node> myNode;
protected:
	IDirect3DDevice9 *device;
	Scene *myScene;
	
	bool nodeAdded;
	int rendererType;
	int myRenderPriority;
};
