#pragma once
#include <boost/shared_ptr.hpp>
#include "Node.h"
#include "Renderer.h"
#include "Object3d.h"
#include "Vector.h"
#include "fmod.h"
//
#include "Scene.h"

#define SPTR boost::shared_ptr

/**
ResourceManager class
*/

class ResourceManager
{

public:
	/**
	Constructor for ResourceManager object
	*/
	ResourceManager(void);
	/**
	Destructor for ResourceManager object
	*/
	~ResourceManager(void);
	
	/**
	Clean up the used resources
	*/
	void cleanUpResources(bool endOfGame);

	/**
	Free reources used by the specified node
	*/
	void freeUsedResources(SPTR<Node> node);
	/**
	Free reources used by the specified renderer
	*/
	void freeUsedResources(SPTR<Renderer> renderer);
	/**
	Set the GameManager
	*/
	void setGameManager(GameManager *manager);

	void OnLostDevice();
	void OnDestroyDevice();

	/**
	Used to load a *.x model
	*/
	bool loadXModel(std::string filename, Object3d *myObject);
	/**
	Used to load a texture
	*/
	IDirect3DTexture9* loadTexture(std::string filename, bool alwaysNeeded = false);
	/**
	Used to load sound
	*/
	FSOUND_SAMPLE* loadSound(std::string filename, bool loadAsLoop);

private:
	struct modelResources
	{
		std::string filename;
		LPD3DXMESH myMesh;
		DWORD NumMaterials;
		DWORD *adjacencyInfo;
		std::vector<D3DMATERIAL9> Materials;
		std::vector<IDirect3DTexture9*> Textures;
		Vector minBBox;
		Vector maxBBox;
	};

	struct textureResources
	{
		std::string filename;
		IDirect3DTexture9* texture;
		bool alwaysNeeded;
	};

	struct soundResources
	{
		std::string filename;
		FSOUND_SAMPLE* soundSample;
		bool loadedForLoop;
	};

    
	GameManager *gameManager;

	std::vector<modelResources> resourceVector;
	std::vector<textureResources> textureVector;
	std::vector<soundResources> soundVector;
};

