#include "dxstdafx.h"
#include ".\scene.h"
#include "GameManager.h"
#include "Object3d.h"
#include "Box.h"
#include "SoundNode.h"
#include "SharedResourceTexture.h"
#include "LowMemLight.h"

#include "fmod.h"

Scene::Scene(void)
{
	this->sceneAlpha = 1;
	this->root.setScene(*this);
	this->sceneLifeTime = 0;

	//Fade Stuff
	this->visible = true;
	this->fadeState = NOFADE;

	//Frustum Stuff
	this->enableFrustumCulling = true;
	this->drawBBoxes = true;
	this->boundingBox = NULL;
	this->quadTreeBuilt = false;

	//RenderChain Timing usw
	QueryPerformanceFrequency(&this->frequency);
	this->firstFrame = true;
	
	//PhysicStuff
	D3DXMatrixIdentity(&this->worldMatrix);
	D3DXMatrixIdentity(&this->identMatrix);
	D3DXMatrixIdentity(&this->lightViewMatrix);
	
	this->root.myScene = this;

	//Get the Device
	this->device = DXUTGetD3DDevice();
	
	//Sun Stuff
	this->sunDirection.setXYZ(-1.0f, -2.0f, 1.0f);
	this->specialRenderer = NULL;

	//DefaultRenderPass konfigurieren
	this->activeRenderPass = NULL;

	this->finalImage = NULL;
	this->preDistortionFinal = NULL;
	this->distortionMap1 = NULL;
	this->distortionMap2 = NULL;
	this->screenDistortionMap = NULL;
	this->quadMesh = NULL;
	this->meshCreated = false;
	this->depthTextureResource = NULL;
	
	this->muteMusic = false;

	this->shadowMapTextureResource = NULL;
	this->shadowMapSurface = NULL;
	currentLight = NULL;

	this->showSoftShadowMaps = true;
	
	filterMode = FILTER_PCF_1D;
	altDown = false;
	this->showDebugInformation = true;
	useTextures = true;
	shadowingMode = Scene::SHMODE_NORMAL;

	sumMode = SUM_ONEPASS;
}

Scene::~Scene(void)
{
	SAFE_RELEASE(this->finalImage);
	SAFE_RELEASE(this->finalImageSurface);
	SAFE_RELEASE(this->quadMesh);
	SAFE_RELEASE(this->depthSurface);
	SAFE_RELEASE(this->preDistortionFinal);
	SAFE_RELEASE(this->preDistortionFinalSurface);
	SAFE_RELEASE(this->screenDistortionMapSurface);
	SAFE_RELEASE(this->shadowMapSurface);
}

void Scene::initScene(GameManager &_manager)
{
	HRESULT hr;
	
	this->manager = &_manager;
	
	//Textur fr endgltiges Bild erzeugen
	if(!this->finalImage) {
		//Create Texture
		V( DXUTGetD3DDevice()->CreateTexture(this->manager->screenWidth, this->manager->screenHeight, 1, D3DUSAGE_RENDERTARGET, 
		D3DFMT_A16B16G16R16F, D3DPOOL_DEFAULT, &this->finalImage, NULL ) );
		//Get Surface for RenderTarget
		this->finalImage->GetSurfaceLevel(0, &this->finalImageSurface);
	}
	
	this->sceneLifeTime = 0;
	this->firstFrame = true;
	
	//this->defaultParticleGroup = (ParticleGroup*) this->createNode(Scene::NODE_PARTICLEGROUP, false);
	this->boundingBox = (Box*) this->createNode(Scene::NODE_BOX);
	this->boundingBox->setVisible(false);
	this->boundingBox->setStandBy(true);
	
	//Camera Stuff
	this->defaultCamera = (Camera *) this->createNode(this->NODE_CAMERA);
	this->activeCamera = this->defaultCamera;
	
	//Default RenderPass
	this->defaultRenderPass.setSafeAndRestoreRenderTarget(false);
	this->defaultRenderPass.setScene(*this);
	this->defaultRenderPass.setRenderPassId(Scene::PASS_NORMAL);
	this->defaultRenderPass.setRenderTarget(this->finalImageSurface);
	this->defaultRenderPass.execute(RenderPass::CMD_CLEARRENDERTARGETANDZBUFFER);
	this->defaultRenderPass.execute(RenderPass::CMD_RENDERNODES);
	
	//Depth RenderPass
	this->depthTextureResource = (SharedResourceTexture*) this->getSharedResource(Scene::SR_TEXTURE_VIEWDEPTH);
	if(!this->depthTextureResource) {
		if(this->depthSurface){
			SAFE_RELEASE (this->depthSurface);
		}
		this->depthTextureResource = (SharedResourceTexture*) this->createSharedResource(Scene::SR_TEXTURE_VIEWDEPTH);
		this->depthTextureResource->getTexture()->GetSurfaceLevel(0, &this->depthSurface);
	}

	this->shadowMapTextureResource = (SharedResourceTexture*) this->getSharedResource(Scene::SR_TEXTURE_SHADOWMAP);
	if(!this->shadowMapTextureResource) {
		if(this->shadowMapSurface){
			SAFE_RELEASE (this->shadowMapSurface);
		}
		this->shadowMapTextureResource = (SharedResourceTexture*) this->createSharedResource(Scene::SR_TEXTURE_SHADOWMAP);
		this->shadowMapTextureResource->getTexture()->GetSurfaceLevel(0, &this->shadowMapSurface);
	}

	this->shadowMapRenderPass.setSafeAndRestoreRenderTarget(false);
	this->shadowMapRenderPass.setScene(*this);
	this->shadowMapRenderPass.setRenderPassId(Scene::PASS_SHADOWMAP);

	this->shadowMapRenderPass.setRenderTarget(this->shadowMapSurface);

	this->shadowMapRenderPass.execute(RenderPass::CMD_CLEARRENDERTARGETANDZBUFFER);
	this->shadowMapRenderPass.execute(RenderPass::CMD_RENDERNODES);
	this->shadowMapRenderPass.execute(RenderPass::CMD_DOFRUSTUMCULLING);

	this->shadowMapRenderPass.useThisRendererList(this->defaultRenderPass.usedRendererList);

	//Initialize SoftShadowRenderpass
	this->softShadowRenderPass.setSafeAndRestoreRenderTarget(false);
	this->softShadowRenderPass.setScene(*this);
	this->softShadowRenderPass.setRenderPassId(Scene::PASS_SOFTSHADOWMAP);
	this->softShadowRenderPass.execute(RenderPass::CMD_CLEARRENDERTARGETANDZBUFFER);
	this->softShadowRenderPass.execute(RenderPass::CMD_RENDERNODES);
	this->softShadowRenderPass.execute(RenderPass::CMD_DOFRUSTUMCULLING);
	this->softShadowRenderPass.setRenderTarget(this->shadowMapSurface);
	this->softShadowRenderPass.useThisRendererList(this->defaultRenderPass.usedRendererList);


	//Load DistortionMaps
	if(!this->distortionMap1)
		this->distortionMap1 = this->manager->resManager.loadTexture("./media/textures/heatnoise.tga");
	if(!this->distortionMap2) 
		this->distortionMap2 = this->manager->resManager.loadTexture("./media/textures/heatnoise2.tga");
	if(!this->screenDistortionMap) {
		SharedResourceTexture* distMap = (SharedResourceTexture*) this->createSharedResource(Scene::SR_TEXTURE_DISTORTION);
		this->screenDistortionMap = distMap->getTexture();
		this->screenDistortionMap->GetSurfaceLevel(0, &this->screenDistortionMapSurface);
	}

	if(!this->preDistortionFinal) {
		//Create Texture
		V( DXUTGetD3DDevice()->CreateTexture(this->manager->screenWidth, this->manager->screenHeight, 1, D3DUSAGE_RENDERTARGET, 
		D3DFMT_A16B16G16R16F, D3DPOOL_DEFAULT, &this->preDistortionFinal, NULL ) );

		//Get Surface for RenderTarget
		this->preDistortionFinal->GetSurfaceLevel(0, &this->preDistortionFinalSurface);
	}

	this->depthRenderPass.setSafeAndRestoreRenderTarget(false);
	this->depthRenderPass.setScene(*this);
	this->depthRenderPass.setRenderPassId(Scene::PASS_DEPTH);
	this->depthRenderPass.setRenderTarget(this->depthSurface);
	this->depthRenderPass.execute(RenderPass::CMD_CLEARRENDERTARGETANDZBUFFER);
	this->depthRenderPass.execute(RenderPass::CMD_RENDERNODES);
	this->depthRenderPass.useThisRendererList(this->defaultRenderPass.usedRendererList);
}

float Scene::getSceneLiveTime()
{
	return this->sceneLifeTime;
}

void Scene::setVisible(bool _visible)
{
	this->visible = _visible;
}

bool Scene::isVisible() 
{
	return this->visible;
}
void Scene::fadeIn(float _duration)
{
	this->fadeDuration = _duration;
	this->fadeTimer = 0;
	this->fadeState = FADEIN;
	this->setVisible(true);
}
void Scene::fadeOut(float _duration)
{
	this->fadeDuration = _duration;
	this->fadeTimer = 0;
	this->fadeState = FADEOUT;
}

void Scene::setSceneAlpha(float _alpha) 
{
	if(_alpha>1) _alpha = 1;
	if(_alpha<0) _alpha = 0;
	this->sceneAlpha = _alpha;
}

Node* Scene::getRoot()
{
	return &this->root;
}

void Scene::clearScene() {
	this->root.removeAllChildren();
	this->cleanUpScene();
	
	this->nodeList.clear();
	this->rendererList.clear();
	this->refObject3dList.clear();
	this->refCameraList.clear();
	this->refSoundList.clear();
	this->triggerList.clear();
	this->clearSharedResources();
	this->sharedResourceVector.clear();
	this->visibleNodeList.clear();
	this->soundNodeList.clear();
	this->softShadowLightList.clear();
	this->zombieNodeList.clear();

}

void Scene::setWidth(float _w)
{
	this->width = _w;
}

void Scene::setHeight(float _h)
{
	this->height = _h;
}

void Scene::setDepth(float _d)
{
	this->depth = _d;
}

float Scene::getWidth()
{
	return this->width;
}

float Scene::getHeight()
{
	return this->height;
}

float Scene::getDepth() 
{
	return this->depth;
}

Camera* Scene::getActiveCamera()
{
	return this->activeCamera;
}

float Scene::getDeltaTime()
{
	return this->dt;
}

void Scene::buildQuadTree()
{
	this->fcQuad.setDimension(this->getWidth(), this->getWidth());
	this->fcQuad.setPosition(this->getWidth()/2, this->getWidth()/2);
	this->fcQuad.buildQuadTree(QUADLEVEL);
	this->quadTreeBuilt = true;
}

RenderPass* Scene::getActiveRenderPass()
{
	return this->activeRenderPass;
}

void Scene::renderScene(float fElapsedTime) 
{
	if(this->isVisible()) {
		if(this->device==0) {
			this->device = DXUTGetD3DDevice();
		}
		this->device->SetRenderTarget(0, this->finalImageSurface);

		this->takeTime();
		//Clear SharedResources
		this->clearSharedResources();
		//Fading Stuff
		this->doFade();
		//Trigger updates
		this->updateTrigger();
		//Update Nodes
		this->updateNodes();
		//Calculate Transformations
		this->calculateTransformations();
		
		//startRenderPasses
		this->startRenderPasses();

		//Cleanup Memory
		this->cleanUpScene();

		this->firstFrame = false;
	}
}

void Scene::takeTime()
{
	//Take time
	if(this->firstFrame) {
		QueryPerformanceCounter(&this->currentFrame);
		this->lastFrame = this->currentFrame;
		return;
	} else {
		this->lastFrame = this->currentFrame;
		QueryPerformanceCounter(&this->currentFrame);
	}
	double tempdt;
	tempdt = (double)(this->currentFrame.QuadPart-this->lastFrame.QuadPart)/(double)this->frequency.QuadPart;
	this->dt = (float) tempdt;	
	this->dt = min(0.033f, this->dt);
}

void Scene::doFade()
{
	switch(this->fadeState) 
	{
		case FADEOUT:
			this->fadeTimer+=this->dt;
			this->sceneAlpha -= this->dt/this->fadeDuration;
			if(this->sceneAlpha<=0) {
				this->setVisible(false);
				this->fadeState = NOFADE;
				this->sceneAlpha = 0;
			}
			this->updateSoundNodeVolume();
			break;
		case FADEIN:
			if(!this->visible)
				this->setVisible(true);
			this->fadeTimer+=this->dt;
			this->sceneAlpha+=this->dt/this->fadeDuration;
			if(this->sceneAlpha>=1) {
				this->fadeState = NOFADE;
				this->sceneAlpha = 1;
			}
			this->updateSoundNodeVolume();
			break;
		default:
			break;
	}
}

void Scene::updateTrigger()
{
	this->sceneLifeTime+=this->getDeltaTime();
	std::vector<SPTR<Trigger> >::iterator it;
	for(it=this->triggerList.begin();it!=this->triggerList.end();) {
		if(this->sceneLifeTime>(*it)->getTime()) {
			this->executeTrigger(*it);
			it = this->triggerList.erase(it);
			//it--;
		} else {
			it++;
			return;
		}
	}

	for(UINT i=0;i<this->triggerList.size();i++) {
		if(this->sceneLifeTime>this->triggerList.at(i)->getTime()) {
			this->executeTrigger(this->triggerList.at(i));
			i--;
		} else {
			return;
		}
	}
}

void Scene::updateNodes() 
{
	this->root.update(this->dt);
}


void Scene::calculateTransformations() 
{
	//Object transformation
	this->worldMatrix = this->identMatrix;
	this->root.calcWorldMatrix(this->worldMatrix);
	this->root.updateBBox();
}



void Scene::doFrustumCulling()
{
	Camera cam(((float)this->manager->screenWidth)/((float)this->manager->screenHeight));
	cam.setViewMatrix(*this->activeRenderPass->getViewMatrix());
	cam.setProjectionMatrix(*this->activeRenderPass->getProjectionMatrix());
    
	std::vector<Renderer *>* usedRendererList;
	usedRendererList = this->activeRenderPass->usedRendererList;

	std::list<SPTR<Node> >::iterator nodeIt;
	Node* node;

	if(!this->quadTreeBuilt)
		this->buildQuadTree();
	usedRendererList->clear();

	Renderer *renderer;

	if(this->enableFrustumCulling) {
		Plane view[4];
		
		view[0] = cam.getNearPlane();
		view[1] = cam.getFarPlane();
		view[2] = cam.getLeftPlane();
		view[3] = cam.getRightPlane();
		this->fcQuad.clipQuadAgainstCamera(&view[0]);

		for(nodeIt = this->nodeList.begin(); nodeIt!=this->nodeList.end();nodeIt++) {
			node = (*nodeIt).get();
			if(!node->onStandBy() && node->visible && node->hasRenderer() &&  node->testAgainstFrustum && this->fcQuad.nodeInside(node)) {	
				renderer = node->getRenderer().get();
				usedRendererList->push_back(renderer);
			} else if(!node->onStandBy() && node->visible && node->hasRenderer() && !node->testAgainstFrustum) {
				renderer = node->getRenderer().get();
				usedRendererList->push_back(renderer);
			}
		}
	} else {	
		for(nodeIt = this->nodeList.begin(); nodeIt!=this->nodeList.end();nodeIt++) {
			node = (*nodeIt).get();
			if(!node->onStandBy() && node->visible && node->hasRenderer()) {	
				renderer = node->getRenderer().get();
				usedRendererList->push_back(renderer);
			}
		}
	}
}

void Scene::doViewingTransformation() 
{
	this->device->SetTransform(D3DTS_VIEW, this->activeCamera->getViewMatrix() );
}


void Scene::renderNodes()
{
	std::vector<Renderer *>* usedRendererList;
	usedRendererList = this->activeRenderPass->usedRendererList;

	//Sorts the rendering processes by its priority!
	std::sort(usedRendererList->begin(), usedRendererList->end(), renderSortFunction());

	std::vector<Renderer *>::iterator it;
	for(it=usedRendererList->begin();it!=usedRendererList->end();it++) {
		if((*it) == 0) {
			this->manager->printToConsole("Renderer in usedRenderList is NULL!!!!");
		}
		if((*it)->nodeAdded) {
			SPTR<Node> n = (*it)->myNode.lock();
			if(!n->onStandBy() && n->visible)
			{
				this->worldMatrix = *n->getWorldMatrix();
				(*it)->render();
			}
		}
	}
}

void Scene::cleanUpScene()
{
	std::list<SPTR<Node> >::iterator it;
	std::list<SPTR<Node> >::iterator nit;
	std::vector<SPTR<Renderer> >::iterator rit;
	Node* node;
	for(it=this->zombieNodeList.begin();it!=this->zombieNodeList.end();it++) {
		node = (*it).get();
		node->getFather()->removeChild(*it);

		//Delete Renderer
		if(node->hasRenderer())
		{
			Renderer *r = node->getRenderer().get();
			for(rit=this->rendererList.begin();rit!=this->rendererList.end();rit++) {
				if((*rit).get() == r)
				{
					this->rendererList.erase(rit);
					break;
				}
			}
		}

		//Delete from NodeList
		this->deleteNodeInList(node, this->nodeList);
		this->deleteNodeInList(node, this->soundNodeList);
		this->deleteNodeInList(node, this->softShadowLightList);
	}
	this->zombieNodeList.clear();
}

void Scene::deleteNodeInList(Node* node, std::list<SPTR<Node> > &list) {
	std::list<SPTR<Node> >::iterator it;
	for(it=list.begin();it!=list.end();it++) {
		if((*it).get() == node)
		{
			list.erase(it);
			break;
		}
	}
}

void Scene::deleteRendererInList(Renderer* renderer) {
	// remove particle renderer
	std::vector<SPTR<Renderer> >::iterator it;
	for(it=rendererList.begin();it!=rendererList.end();it++) {
		if((*it).get() == renderer) {
			rendererList.erase(it);
			break;
		}
	}
}

void Scene::clearRenderTargetAndZBuffer()
{
	if(getActivePassId() == Scene::PASS_SHADOWMAP) {
		DXUTGetD3DDevice()->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_ARGB(255, 255, 255, 255), 1.0f, 0);
	} else {
		DXUTGetD3DDevice()->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, D3DCOLOR_ARGB(255, 0, 0, 0), 1.0f, 0);
	}
}

void Scene::clearSharedResources()
{
	for(UINT i=0;i<this->sharedResourceVector.size();i++) {
		this->sharedResourceVector.at(i)->setResourceAvailable(false);
	}
}

void Scene::setKeyPressed(UINT key, bool bKeyPressed, bool bAltDown) 
{
	this->keyDown[key] = bKeyPressed;
	this->altDown = bAltDown;
}

void Scene::setMouseStatus(bool bLeftButtonDown,  bool bRightButtonDown,bool bMiddleButtonDown,
						bool bSideButton1Down, bool bSideButton2Down, int nMouseWheelDelta,
						int xPos, int yPos)
{
	this->mouseButton[MOUSE_LEFT]	= bLeftButtonDown;
	this->mouseButton[MOUSE_RIGHT]	= bRightButtonDown;
	this->mouseButton[MOUSE_MIDDLE] = bMiddleButtonDown;
	this->mouseButton[MOUSE_SIDE1] = bSideButton1Down;
	this->mouseButton[MOUSE_SIDE2]	= bSideButton2Down;
	this->mousePos[0] = xPos;
	this->mousePos[1] = yPos;
	this->mouseWheel = nMouseWheelDelta;
}

Node* Scene::createNode(int type)
{
	return this->createNode(type, this->root, true, false);
}

Node* Scene::createNode(int type, Node &father)
{
	return this->createNode(type, father, true, false);
}

Node* Scene::createNode(int type, bool addDefaultRenderer)
{
	return this->createNode(type, this->root, addDefaultRenderer, false);
}

Node* Scene::createNode(int type, Node &father, bool addDefaultRenderer) {
	return this->createNode(type, father, addDefaultRenderer, false);
}

Node* Scene::createReferenceNode(int type, bool addDefaultRenderer)
{
	Node* n=0;
	Node* newNode = this->createNode(type, *n, addDefaultRenderer, true);
	newNode->setStandBy(true);
	return newNode;
}

Node* Scene::createNode(int type, Node &father, bool addDefaultRenderer, bool isReference)
{
    switch(type) 
	{
		case NODE_OBJECT3D:
			{
				SPTR<Node> node(new Object3d);
				node->setScene(*this);
				if(!isReference) {
					father.addChild(node);
					this->nodeList.push_back(node);
				} else {
					this->refObject3dList.push_back(node);
				}
				if(addDefaultRenderer) {
					SPTR<Renderer> renderer(new SimpleMeshRenderer);
					renderer->setScene(*this);
					WPTR<Node> wn(node);
					renderer->setNode(wn);
					node->setRenderer(renderer);
					renderer->init();
					this->rendererList.push_back(renderer);
				}
				return node.get(); 
			}
			break;
		case NODE_CAMERA:
			{
				float ratio = ((float)this->manager->screenWidth)/((float)this->manager->screenHeight);
				SPTR<Node> node(new Camera(ratio));
				if(!isReference) {
					father.addChild(node);
					this->nodeList.push_back(node);
				} else {
					this->refCameraList.push_back(node);
				}
				node->setScene(*this);
				
				return node.get();
			}
			break;
		case NODE_SOUND:
			{
				SPTR<Node> node(new SoundNode);
				if(!isReference) {
					father.addChild(node);
					this->nodeList.push_back(node);
					this->soundNodeList.push_back(node);
				} else {
					this->refSoundList.push_back(node);
				}
				node->setScene(*this);
				
				return node.get(); 
			}
		case NODE_BOX:
			{
				SPTR<Node> node(new Box);
				node->setScene(*this);
				if(!isReference) {
					father.addChild(node);
					this->nodeList.push_back(node);
				}
				if(addDefaultRenderer) {
					SPTR<Renderer> renderer(new SimpleMeshRenderer);
					renderer->setScene(*this);
					WPTR<Node> wn(node);
					renderer->setNode(wn);
					node->setRenderer(renderer);
					renderer->init();
					this->rendererList.push_back(renderer);
				}
				return node.get(); 
			}
			break;
		case NODE_SOFTSHADOWLOWMEMLIGHT:
			{
				SPTR<Node> node(new LowMemLight);
				father.addChild(node);
				this->nodeList.push_back(node);
				node->setScene(*this);
				Node *temp = node.get();
				if(addDefaultRenderer) {
					SPTR<Renderer> renderer(new SoftShadowLightRenderer);
					renderer->setScene(*this);
					WPTR<Node> wn(node);
					renderer->setNode(wn);
					node->setRenderer(renderer);
					renderer->init();
					this->rendererList.push_back(renderer);
				}
				this->softShadowLightList.push_back(node);
				return temp;
			}
			break;
	}
	return 0;
}

void Scene::connectNodeAndRenderer(Node &node, SPTR<Renderer> &renderer)
{
	std::list<SPTR<Node> >::iterator it;
	SPTR<Node> sN;
	for(it = this->nodeList.begin(); it!=this->nodeList.end();it++) {
		if((*it).get() == &node) {
			WPTR<Node> wn(*it);
			renderer->setScene(*this);
			renderer->setNode(wn);
			(*it)->setRenderer(renderer);
			renderer->init();
			this->rendererList.push_back(renderer);
			break;
		}
	}
}

void Scene::OnLostDevice( void* pUserContext )
{
	SAFE_RELEASE(this->finalImage);
	SAFE_RELEASE(this->finalImageSurface);
	SAFE_RELEASE(this->quadMesh);
	SAFE_RELEASE(this->depthSurface);
	SAFE_RELEASE(this->preDistortionFinal);
	SAFE_RELEASE(this->preDistortionFinalSurface);
	SAFE_RELEASE(this->screenDistortionMapSurface);
	
	SAFE_RELEASE(this->shadowMapSurface);

	this->finalImage = NULL;
	this->finalImageSurface = NULL;
	this->meshCreated = false;
	this->quadMesh = NULL;
	this->preDistortionFinal = NULL;
	this->preDistortionFinalSurface = NULL;
	this->screenDistortionMapSurface = NULL;

	std::list<SPTR<Node> >::iterator nit;
	for(nit=this->nodeList.begin();nit!=this->nodeList.end();nit++) {
		(*nit)->OnLostDevice(pUserContext);
	}

	for(UINT i=0;i<this->sharedResourceVector.size();i++) {
		this->sharedResourceVector.at(i)->OnLostDevice();
	}

	std::vector<SPTR<Renderer> >::iterator it;
	for(it=this->rendererList.begin();it!=this->rendererList.end();it++) {
		(*it)->OnLostDevice(pUserContext);
	}
}

void Scene::OnDestroyDevice( void* pUserContext )
{
	std::list<SPTR<Node> >::iterator nit;
	for(nit=this->nodeList.begin();nit!=this->nodeList.end();nit++) {
		(*nit)->OnDestroyDevice(pUserContext);
	}

	std::vector<SPTR<Renderer> >::iterator it;
	for(it=this->rendererList.begin();it!=this->rendererList.end();it++) {
		(*it)->OnDestroyDevice(pUserContext);
	}

	this->depthTextureResource = NULL;
	this->sharedResourceVector.clear();
	this->distortionMap1 = NULL;
	this->distortionMap2 = NULL;
	this->screenDistortionMap = NULL;
	this->preDistortionFinal = NULL;

	this->shadowMapTextureResource = NULL;
}

HRESULT Scene::OnResetDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
	HRESULT hr;

	//Restore that stuff
	this->device = pd3dDevice;

	SAFE_RELEASE(this->quadMesh);
	SAFE_RELEASE(this->depthSurface);
	SAFE_RELEASE(this->preDistortionFinal);
	SAFE_RELEASE(this->preDistortionFinalSurface);
	SAFE_RELEASE(this->screenDistortionMapSurface);
	
	SAFE_RELEASE(this->shadowMapSurface);

	//Textur fr endgltiges Bild erzeugen
	if(!this->finalImage) {
		//Create Texture
		V( DXUTGetD3DDevice()->CreateTexture(this->manager->screenWidth, this->manager->screenHeight, 1, D3DUSAGE_RENDERTARGET, 
		D3DFMT_A16B16G16R16F, D3DPOOL_DEFAULT, &this->finalImage, NULL ) );

		//Get Surface for RenderTarget
		this->finalImage->GetSurfaceLevel(0, &this->finalImageSurface);
	}

	//Depth RenderPass
	this->depthTextureResource = (SharedResourceTexture*) this->getSharedResource(Scene::SR_TEXTURE_VIEWDEPTH);
	if(!this->depthTextureResource) {
		this->depthTextureResource = (SharedResourceTexture*) this->createSharedResource(Scene::SR_TEXTURE_VIEWDEPTH);
	}
	this->depthTextureResource->getTexture()->GetSurfaceLevel(0, &this->depthSurface);

	//Shadow Map RenderPass
	this->shadowMapTextureResource = (SharedResourceTexture*) this->getSharedResource(Scene::SR_TEXTURE_SHADOWMAP);
	if(!this->shadowMapTextureResource) {
		this->shadowMapTextureResource = (SharedResourceTexture*) this->createSharedResource(Scene::SR_TEXTURE_SHADOWMAP);
	}
	this->shadowMapTextureResource->getTexture()->GetSurfaceLevel(0, &this->shadowMapSurface);

	//Load DistortionMaps
	if(!this->distortionMap1)
		this->distortionMap1 = this->manager->resManager.loadTexture("./media/textures/heatnoise.tga");
	if(!this->distortionMap2) 
		this->distortionMap2 = this->manager->resManager.loadTexture("./media/textures/heatnoise2.tga");
	if(!this->screenDistortionMap) {
		SharedResourceTexture* distMap = (SharedResourceTexture*) this->createSharedResource(Scene::SR_TEXTURE_DISTORTION);
		this->screenDistortionMap = distMap->getTexture();
		this->screenDistortionMap->GetSurfaceLevel(0, &this->screenDistortionMapSurface);
	}

	if(!this->preDistortionFinal) {
		//Create Texture
		V( DXUTGetD3DDevice()->CreateTexture(this->manager->screenWidth, this->manager->screenHeight, 1, D3DUSAGE_RENDERTARGET, 
		D3DFMT_A16B16G16R16F, D3DPOOL_DEFAULT, &this->preDistortionFinal, NULL ) );

		//Get Surface for RenderTarget
		this->preDistortionFinal->GetSurfaceLevel(0, &this->preDistortionFinalSurface);
	}

	this->defaultRenderPass.setRenderTarget(this->finalImageSurface);
	this->depthRenderPass.setRenderTarget(this->depthSurface);

	std::list<SPTR<Node> >::iterator nit;
	for(nit=this->nodeList.begin();nit!=this->nodeList.end();nit++) {
		(*nit)->OnResetDevice(pd3dDevice, pBackBufferSurfaceDesc, pUserContext);
	}

	std::vector<SPTR<Renderer> >::iterator it;
	for(it=this->rendererList.begin();it!=this->rendererList.end();it++) {
		(*it)->OnResetDevice(pd3dDevice, pBackBufferSurfaceDesc, pUserContext);
	}
	return S_OK;
}

Vector Scene::getSunDirection() 
{
	return this->sunDirection;
}

void Scene::setSunDirection(Vector &v)
{
	this->sunDirection.x = v.x;
	this->sunDirection.y = v.y;
	this->sunDirection.z = v.z;
	this->sunDirection.normalize();
}

Vector Scene::getSunPosition() 
{
	return this->sunPosition;
}

void Scene::setSunPosition(Vector &v)
{
	this->sunPosition.x = v.x;
	this->sunPosition.y = v.y;
	this->sunPosition.z = v.z;
}

Vector Scene::getSunLookAtTarget() 
{
	return this->sunLookAtTarget;
}

void Scene::setSunLookAtTarget(Vector &v)
{
	this->sunLookAtTarget.x = v.x;
	this->sunLookAtTarget.y = v.y;
	this->sunLookAtTarget.z = v.z;
}

bool Scene::calculateLightViewMatrix() {
	D3DXVECTOR3 position = this->getSunPosition().getD3DXVector3();
	D3DXVECTOR3 lookAtTarget = this->getSunLookAtTarget().getD3DXVector3();
	D3DXVECTOR3 upVector = Vector(0, 1, 0);//this->defaultCamera->getUpVector();

	if(position == 0 || lookAtTarget == 0 || upVector == 0)
		return false;
		
	D3DXMatrixLookAtLH(&this->lightViewMatrix, &position, &lookAtTarget, &upVector);

	return true;
}

D3DXMATRIX Scene::getIdentityMatrix()
{
	return this->identMatrix;
}

D3DXMATRIX Scene::getWorldMatrix()
{
	return this->worldMatrix;
}

D3DXMATRIX Scene::getViewMatrix()
{
	return *this->activeRenderPass->getViewMatrix();
}

D3DXMATRIX Scene::getProjectionMatrix()
{
	return *this->activeRenderPass->getProjectionMatrix();
}

D3DXMATRIX Scene::getLightViewMatrix()
{
	return this->lightViewMatrix;
}

D3DXMATRIX Scene::getLightReferenceViewMatrix()
{
	return this->lightRefViewMatrix;
}

void Scene::setTrigger(int type, float time)
{
	SPTR<Trigger> t(new Trigger(type, time));
	this->triggerList.push_back(t);
	std::sort(this->triggerList.begin(), this->triggerList.end(), triggerSortFunction());
}

void Scene::setTrigger(int type, float time, std::string text)
{
	SPTR<Trigger> t(new Trigger(type, time, text));
	this->triggerList.push_back(t);
	std::sort(this->triggerList.begin(), this->triggerList.end(), triggerSortFunction());
}

void Scene::setTrigger(int type, Node *node, float time)
{
	SPTR<Node> sN = this->getSmartPointer(node);
	
	SPTR<Trigger> t(new Trigger(type, sN, time));
	this->triggerList.push_back(t);
	std::sort(this->triggerList.begin(), this->triggerList.end(), triggerSortFunction());
}

void Scene::setTrigger(int type, Node *node, Node* secondNode, float time)
{
	SPTR<Node> sN = this->getSmartPointer(node);
	SPTR<Node> secondSN = this->getSmartPointer(secondNode);
	
	SPTR<Trigger> t(new Trigger(type, sN, secondSN, time));
	this->triggerList.push_back(t);
	std::sort(this->triggerList.begin(), this->triggerList.end(), triggerSortFunction());
}

void Scene::setTrigger(int type, Node *node, Node* secondNode, Vector normal, float time)
{
	SPTR<Node> sN = this->getSmartPointer(node);
	SPTR<Node> secondSN = this->getSmartPointer(secondNode);
	
	SPTR<Trigger> t(new Trigger(type, sN, secondSN, normal, time));
	this->triggerList.push_back(t);
	std::sort(this->triggerList.begin(), this->triggerList.end(), triggerSortFunction());
}

SPTR<Node> Scene::getSmartPointer(Node *node)
{
	std::list<SPTR<Node> >::iterator it;
	for(it = this->nodeList.begin(); it!=this->nodeList.end();it++) {
		if((*it).get() == node) {
			break;
		}
	}
	if(it==this->nodeList.end()) {
		this->manager->printToConsole("NODE NOT FOUND IN LIST!!!");
	}
	return (*it);
}

void Scene::executeTrigger(SPTR<Trigger> trigger)
{
	Trigger *t = trigger.get();
	switch(t->getType())
	{
		case this->TRIGGER_UNSETSTANDBY:
			if(!t->getNode()->isA(Scene::NODE_SOUND)) {
				t->getNode()->setStandBy(false);
			} else {
				t->getNode()->setStandBy(false);
				((SoundNode*) t->getNode().get())->play();
			}
			break;
		case this->TRIGGER_KILLNODE:
			t->getNode()->killMe();
			break;
		case this->TRIGGER_SETDETAIL:
			break;
	}
}

void Scene::deleteNode(Node* node)
{
	std::list<SPTR<Node> >::iterator it;
	for(it=this->zombieNodeList.begin();it!=this->zombieNodeList.end();it++) {
		if((*it).get() == node) {
			return;
		}
	}
	SPTR<Node> sn = this->getSmartPointer(node);
	this->zombieNodeList.push_back(sn);
}

void Scene::setLight(Vector direction) {
	//set ambient light:
	DXUTGetD3DDevice()->SetRenderState( D3DRS_AMBIENT, D3DCOLOR_XRGB(120,120,120) );

	// Fill in a light structure defining our light
	D3DLIGHT9 light;
	ZeroMemory( &light, sizeof(D3DLIGHT9) );
	light.Type       = D3DLIGHT_DIRECTIONAL;
	light.Diffuse.r  = 1.0f;
	light.Diffuse.g  = 1.0f;
	light.Diffuse.b  = 1.0f;
	light.Diffuse.a  = 1.0f;
	light.Range      = 1000.0f;

	// Create a direction for our light - it must be normalized  
	D3DXVECTOR3 vecDir;
	vecDir = D3DXVECTOR3(direction.x,direction.y,direction.z);
	D3DXVec3Normalize( (D3DXVECTOR3*)&light.Direction, &vecDir );

	// Tell the device about the light and turn it on
	DXUTGetD3DDevice()->SetLight( 0, &light );
	DXUTGetD3DDevice()->LightEnable( 0, TRUE );
}

SharedResource* Scene::createSharedResource(int id)
{
	switch(id) {
		case this->SR_TEXTURE_SHADOWMAP:
			{
				SPTR<SharedResource> srt(new SharedResourceTexture);
				SharedResourceTexture* t = (SharedResourceTexture*) srt.get();
				t->setResourceIdentifier(id);
				t->createTexture(this->manager->screenWidth, this->manager->screenHeight, D3DFMT_R32F, D3DUSAGE_RENDERTARGET, D3DPOOL_DEFAULT);
				this->sharedResourceVector.push_back(srt);
				return t;
			}
			break;
		case this->SR_TEXTURE_VIEWDEPTH:
			{
				SPTR<SharedResource> srt(new SharedResourceTexture);
				SharedResourceTexture* t = (SharedResourceTexture*) srt.get();
				t->setResourceIdentifier(id);
				t->createTexture(this->manager->screenWidth, this->manager->screenHeight, D3DFMT_R32F, D3DUSAGE_RENDERTARGET, D3DPOOL_DEFAULT);
				this->sharedResourceVector.push_back(srt);
				return t;
			}
			break;
		case this->SR_TEXTURE_DISTORTION:
			{
				SPTR<SharedResource> srt(new SharedResourceTexture);
				SharedResourceTexture* t = (SharedResourceTexture*) srt.get();
				t->setResourceIdentifier(id);
				t->createTexture(this->manager->screenWidth, this->manager->screenHeight, D3DFMT_A16B16G16R16F, D3DUSAGE_RENDERTARGET, D3DPOOL_DEFAULT);
				this->sharedResourceVector.push_back(srt);
				return t;
			}
			break;
	}
	return NULL;
}

SharedResource* Scene::getSharedResource(int id)
{
	for(UINT i=0;i<this->sharedResourceVector.size();i++) {
		if(this->sharedResourceVector.at(i)->getResourceIdentifier()==id) {
			SharedResource* sr = this->sharedResourceVector.at(i).get();
			return sr;
		}
	}
	return NULL;
}

int* Scene::getMousePos() {
	return this->mousePos;
}

void Scene::OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext )
{
}

void Scene::createQuadMesh()
{	
	HRESULT hr;
	//Create a quad for rendering to screen
	if(SUCCEEDED( D3DXCreateMeshFVF(2, 4, D3DXMESH_MANAGED, this->FVF_Flags, this->device, &this->quadMesh)) ) {
		//lock vertex buffer
		Vector ecken[4];
		Vertex* pVertices;
		V( this->quadMesh->LockVertexBuffer( 0, (void**)&pVertices ) );
		
		ecken[0] = Vector(-1.0f, 1.0f, 0.0f);
		ecken[1] = Vector(-1.0f, -1.0f, 0.0f);
		ecken[2] = Vector(1.0f, -1.0f, 0.0f);
		ecken[3] = Vector(1.0f, 1.0f, 0.0f);

		pVertices[0] = Vertex(ecken[0], 0.0f, 0.0f); 
		pVertices[1] = Vertex(ecken[1], 0.0f, 1.0f);
		pVertices[2] = Vertex(ecken[2], 1.0f, 1.0f);
		pVertices[3] = Vertex(ecken[3], 1.0f, 0.0f);

		//unlock vertex buffer
		V( this->quadMesh->UnlockVertexBuffer() );	

		// fill the indices:
		WORD* pIndices = NULL;
		this->quadMesh->LockIndexBuffer( 0, (void**)&pIndices );

		pIndices[0] = (WORD) 0; pIndices[1] = (WORD) 2; pIndices[2] = (WORD) 1;
		pIndices[3] = (WORD) 0; pIndices[4] = (WORD) 3; pIndices[5] = (WORD) 2;

		this->quadMesh->UnlockIndexBuffer();
		this->meshCreated=true;
	}  else {
		this->manager->printToConsole("ERROR creating mesh!");
	}
	
}

void Scene::renderFinalImage(ID3DXEffect* finalEffect)
{
	HRESULT hr = 0;
	
	
	if(!this->meshCreated) {
		this->createQuadMesh();
	}

	UINT cPass;
	V( finalEffect->SetFloat("alpha", this->sceneAlpha) );
	V( finalEffect->SetTexture("sceneRendering", this->finalImage) );
	V( finalEffect->CommitChanges() );
	V( finalEffect->SetTechnique("finalRendering") );
	V( finalEffect->Begin( &cPass, 0 ) );
	V( finalEffect->BeginPass( 0 ) );
	V( this->device->BeginScene() );
	V( this->device->SetTexture(0, this->finalImage) );
	V( this->quadMesh->DrawSubset(0) );
	V( this->device->EndScene() );
	V( finalEffect->EndPass() );
	V( finalEffect->End() );
}

int Scene::getActivePassId() {
	if(this->activeRenderPass==NULL) {
		return this->PASS_NORMAL;
	} else {
		return this->activeRenderPass->getRenderPassId();
	}
}

void Scene::setActiveRenderPass(RenderPass &_activePass)
{
	this->activeRenderPass = &_activePass;
	this->activeRenderPass->calculateMatrices();
}

void Scene::startRenderPasses()
{
	bool tempDistortionPass = false;

	//Reset values
	this->needDepthPass = false;
	this->needDistortionPass = false;

	this->needShadowMapPass = true;

	if(!this->meshCreated) {
		this->createQuadMesh();
	}

	//DefaultRenderPass aktivieren
	this->defaultRenderPass.setViewMatrix(*this->activeCamera->getViewMatrix());
	this->defaultRenderPass.setProjectionMatrix(*this->activeCamera->getProjectionMatrix());

	//DefaultRenderPass aktivieren
	this->defaultRenderPass.setRenderTarget(this->preDistortionFinalSurface);
	this->setActiveRenderPass(this->defaultRenderPass);
	this->doFrustumCulling();
	this->activeRenderPass->executePass();
	
	//ViewDepthPass rendern
	this->depthRenderPass.setViewMatrix(*this->activeCamera->getViewMatrix());
	this->depthRenderPass.setProjectionMatrix(*this->activeCamera->getProjectionMatrix());
	this->setActiveRenderPass(this->depthRenderPass);
	this->activeRenderPass->executePass();
	this->depthTextureResource->setResourceAvailable(true);

	//SoftShadow Renderpass Execution
	std::list<SPTR<Node> >::iterator softShadowIt;
	
	bool originalEnableFrustumCulling = this->enableFrustumCulling;
	this->enableFrustumCulling = false;
	this->setActiveRenderPass(this->softShadowRenderPass);
	//For each SoftShadowLight calculate Depthpasses!
	for(softShadowIt = this->softShadowLightList.begin(); softShadowIt!=this->softShadowLightList.end();softShadowIt++) {
		currentLight = (LowMemLight*) softShadowIt->get();
		this->softShadowRenderPass.setActiveLight(currentLight);
		this->softShadowRenderPass.executePass();
	}
	this->enableFrustumCulling = originalEnableFrustumCulling;	

	//Render final SoftShadow
	this->doFinalSoftShadowPass();
	
	if(this->showDebugInformation) {
		this->renderShowShadowMaps();
	}

	FSOUND_Update();	//Update sound because of jittering
}

void Scene::updateSoundNodeVolume()
{
	std::list<SPTR<Node> >::iterator it;
	for(it = this->soundNodeList.begin();it!=this->soundNodeList.end();it++) {
		((SoundNode*) it->get())->setSceneAlpha(this->sceneAlpha);
	}
	if(this->bgSoundChannel!=-1 && !this->muteMusic) {
		FSOUND_SetVolume(this->bgSoundChannel, (int)(255*this->bgSoundVolume*this->sceneAlpha));
	}
}

LPD3DXMESH Scene::getQuadMesh()
{
	if(!this->meshCreated) {
		this->createQuadMesh();
	}
	return this->quadMesh;
}

void Scene::setBackgroundSoundMute(bool _muteMusic)
{	
	this->muteMusic = _muteMusic;
	if(!this->muteMusic) {
		if(this->bgSoundChannel!=-1) {
			FSOUND_SetVolume(this->bgSoundChannel, (int)(255*this->bgSoundVolume*this->sceneAlpha));
		}
	} else {
		if(this->bgSoundChannel!=-1) {
			FSOUND_SetVolume(this->bgSoundChannel, 0);
		}
	}
}

bool Scene::getBackgroundSoundMute() {
	return this->muteMusic;
}

void Scene::setBackgroundSound(std::string filename) {
	this->bgMusicStream = FSOUND_Stream_Open(filename.c_str(), FSOUND_LOOP_NORMAL, 0, 0);
}

void Scene::setBackgroundSoundVolume(float vol) {
	this->bgSoundVolume = vol;
	if(this->bgSoundChannel!=-1) {
		if(!this->muteMusic) {
			FSOUND_SetVolume(this->bgSoundChannel, (int)(255*this->bgSoundVolume));
		} else {
			FSOUND_SetVolume(this->bgSoundChannel, 0);
		}
	}
}

void Scene::playBackgroundSound() {
	if(this->bgMusicStream!=NULL) {
		this->bgSoundChannel = FSOUND_Stream_Play(FSOUND_FREE, this->bgMusicStream);
		if(!this->muteMusic) {
			FSOUND_SetVolume(this->bgSoundChannel, (int)(255*this->bgSoundVolume));
		} else {
			FSOUND_SetVolume(this->bgSoundChannel, 0);
		}
	}
}

void Scene::setLightViewMatrix(D3DXMATRIX _lightViewMatrix)
{
	this->lightViewMatrix = _lightViewMatrix;
}

void Scene::setLightReferenceViewMatrix(D3DXMATRIX _lightRefViewMatrix)
{
	this->lightRefViewMatrix = _lightRefViewMatrix;
}

std::list<SPTR<Node> >* Scene::getSoftShadowLightList() {
	return &softShadowLightList;
}

void Scene::doFinalSoftShadowPass()
{
	HRESULT hr;
	UINT cPass;
	ID3DXEffect* shadowEffect;
	shadowEffect = this->manager->getEffect(GameManager::EFFECT_SHADOWMAP);


	this->device->SetRenderTarget(0, this->finalImageSurface);

	switch (sumMode) {
		case SUM_ONEPASS:
			V( shadowEffect->SetTechnique("Combine") );
			break;
		case SUM_WHOLESCENE:
			V( shadowEffect->SetTechnique("AdvancedCombine") );
			break;
		case SUM_TWOPASS:
			V( shadowEffect->SetTechnique("Combine") );
			break;
	}

	V( shadowEffect->SetTexture("g_txSourceMap", this->preDistortionFinal) );
	V( shadowEffect->SetTexture("g_txDepthMap", this->depthTextureResource->getTexture()) );

	std::list<SPTR<Node> >::iterator softShadowIt;
	for(softShadowIt = this->softShadowLightList.begin(); softShadowIt!=this->softShadowLightList.end();softShadowIt++) {
		currentLight = (LowMemLight*) softShadowIt->get();
		//setting the summed up shadow texture (in view space)
		V( shadowEffect->SetFloat("invMaxShadowMaps", 1.0f/currentLight->getNbShadowMaps()) );
		V( shadowEffect->SetTexture("g_txShadowMap", currentLight->getLightSumTexture() ) );
	}

	V( shadowEffect->CommitChanges() );

	V( shadowEffect->Begin( &cPass, 0 ) );
	V( shadowEffect->BeginPass( 0 ) );
	V( this->device->BeginScene() );
	
	V( this->getQuadMesh()->DrawSubset(0) );

	V( this->device->EndScene() );
	V( shadowEffect->EndPass() );
	V( shadowEffect->End() );
}

LowMemLight* Scene::getCurrentLight()
{
	return currentLight;
}

void Scene::renderShowShadowMaps()
{
	HRESULT hr;
	this->device->SetRenderTarget(0, this->finalImageSurface);
	UINT cPass;
	ID3DXEffect* shadowEffect;
	shadowEffect = this->manager->getEffect(GameManager::EFFECT_SHADOWMAP);
	this->device->SetRenderState(D3DRS_ZENABLE, D3DZB_FALSE);
	this->device->SetRenderState(D3DRS_ZWRITEENABLE, FALSE);
	V( shadowEffect->SetTechnique("ShowDepthMaps") );
	V( shadowEffect->Begin( &cPass, 0 ) );
	V( shadowEffect->BeginPass( 0 ) );
	V( this->device->BeginScene() );

	//Render SoftShadowMaps of each LightSource
	float scale;
	float xypos[2];
	int lightCounter=0;
	int si = 0;
	int nbDepthMaps = softShadowRenderPass.maxUsedMaps;
	for(int si = 0;si<this->softShadowRenderPass.getNbDepthMaps();si++) {
		scale = 0.1f;//0.85f/((nbDepthMaps+1));
		xypos[0] = (1-scale*1.2f) - 2.2f*scale*lightCounter;
		xypos[1] = (1-scale*1.2f) - si*scale*2.2f;
		
		V( shadowEffect->SetFloatArray("depthMapPos", xypos, 2) );
		V( shadowEffect->SetFloat("depthMapScale", scale) );
		V( shadowEffect->SetTexture("g_txDepthMap", softShadowRenderPass.getDepthMapAt(si)) );
		V( shadowEffect->CommitChanges() );
		
		V( this->quadMesh->DrawSubset(0) );
	}
	
	//Render SumMesh
	xypos[0] = -0.3f;
	xypos[1] = -0.7f;
	scale = 0.15f;
	V( shadowEffect->SetFloatArray("depthMapPos", xypos, 2) );
	V( shadowEffect->SetFloat("depthMapScale", scale) );
	V( shadowEffect->SetTexture("g_txDepthMap", this->currentLight->getLightSumTexture()) );
	V( shadowEffect->CommitChanges() );
	
	V( this->quadMesh->DrawSubset(0) );

	V( this->device->EndScene() );
	V( shadowEffect->EndPass() );
	V( shadowEffect->End() );
	this->device->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
	this->device->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);
}

IDirect3DTexture9* Scene::getViewDepthMap()
{
	return this->depthTextureResource->getTexture();
}