#include "dxstdafx.h"
#include ".\ShadowSceneloader.h"
#include "GameManager.h"
#include "SkyBox.h"
#include "Object3d.h"
#include "SoundNode.h"
#include "LowMemLight.h"

ShadowSceneLoader::ShadowSceneLoader(void)
{
	
	//XML Stuff
	::CoInitialize(NULL);

	HRESULT hr = sceneDomDocument.CreateInstance(MSXML2::CLSID_DOMDocument);
	if (FAILED(hr))	{
		this->scene->manager->printToConsole("Unable to create XML instance!");
		exit(0);
	}
}

ShadowSceneLoader::~ShadowSceneLoader(void)
{
}

bool ShadowSceneLoader::loadSoftShadowScene(SoftShadowScene &_scene, std::string filename, GameManager *_manager) { 
	this->scene = &_scene;
	// specify xml file name
	variant_t vResult;
	
	vResult = sceneDomDocument->load(filename.c_str());
	if (((bool)vResult) == TRUE) {
		sceneDocRoot = sceneDomDocument->documentElement;
	} else {
		this->scene->manager->printToConsole(filename);
		this->scene->manager->printToConsole("Loading SoftShadowScene failed!");
		MessageBox(NULL, L"Maybe the file is not a valid xml!", L"Error loading .xml file!", MB_ICONERROR);
		return false;
	}
	
	//BuildGame
	_bstr_t tmpSun("sun");
	_bstr_t	tmpDimension("dimension");
	_bstr_t tmpSkyBox("skybox");
	_bstr_t tmpParticleEmitter("particleEmitter");
	_bstr_t tmpSoundFile("soundfile");
	_bstr_t tmpSoundVolume("soundvolume");
	_bstr_t tmpEnvObject("envObject");
	_bstr_t tmpDescriptionDuration("descduration");
	_bstr_t tmpSoftShadowLight("softShadowLight");
	float descTime = 3;
	Vector dimension;

	//this is to ensure that the scene has constant size
	dimension.addXYZ(320, 40, 320);
	this->scene->setWidth(dimension.x);
	this->scene->setHeight(dimension.y);
	this->scene->setDepth(dimension.z);

	for (MSXML2::IXMLDOMNodePtr pChild = sceneDocRoot->firstChild;NULL != pChild;pChild = pChild->nextSibling) {
		if(pChild->nodeType == MSXML2::NODE_ELEMENT) {
			if(pChild->nodeName == tmpSkyBox) {
				this->traceSkyBox(pChild);									//SkyBox
			} else if(pChild->nodeName == tmpSun) {
				this->traceSun(pChild);										//Sun
			} else if(pChild->nodeName == tmpEnvObject) {
				this->traceEnvironmentObject(pChild);						//Environment Object
			} else if(pChild->nodeName == tmpSoundFile) {
				std::string soundFile = pChild->firstChild->text;
				this->scene->setBackgroundSound(soundFile);					//BackgroundSound
			} else if(pChild->nodeName == tmpSoundVolume) {
				this->scene->setBackgroundSoundVolume((FLOAT)atof(pChild->firstChild->text));	//SoundVolume
			} else if(pChild->nodeName == tmpDescriptionDuration) {
				descTime = (FLOAT) atof(pChild->firstChild->text);
			} else if(pChild->nodeName == tmpSoftShadowLight) {
				this->traceSoftShadowLight(pChild);									//SoftShadowLight
			} 
		}
	}

	return true;
}

void ShadowSceneLoader::traceSun(MSXML2::IXMLDOMNodePtr sunNode) {
	Vector direction;
	Vector terrainCenter;
	Vector sunTranslation;

	_bstr_t tmpDirection("direction");

	float distance;
	_bstr_t tmpDistance("distance");

	for (MSXML2::IXMLDOMNodePtr pChild = sunNode->firstChild;NULL != pChild;pChild = pChild->nextSibling) {
		if(pChild->nodeType == MSXML2::NODE_ELEMENT) {
			if(pChild->nodeName == tmpDirection) {
				direction = tracePosition(pChild);
			}
			else if(pChild->nodeName == tmpDistance) {
				distance = (FLOAT)atof(pChild->firstChild->text);
			}
		}
	}

	direction = direction*(-1); //Use direction vector
	this->scene->setLight(direction);

	//get center of the map as light source's lookat target
	float terrainCenterX = this->scene->getWidth()/2.0f;
	float terrainCenterZ = this->scene->getDepth()/2.0f;
	terrainCenter = Vector(terrainCenterX, 0.0f, terrainCenterZ);
	this->scene->setSunLookAtTarget(terrainCenter); 

	//normalize the light direction vector
	//and multiply it by the distance to the lookat target
	direction.normalize();
	sunTranslation = direction * distance;
	this->scene->setSunPosition(terrainCenter + sunTranslation);

	//calculate the matrix one:
	if(!this->scene->calculateLightViewMatrix())
		this->scene->manager->printToConsole("error initializing LightViewMatrix!");
}

Vector ShadowSceneLoader::tracePosition(MSXML2::IXMLDOMNodePtr positionNode) {
	_bstr_t tmpX("x");
	_bstr_t tmpY("y");
	_bstr_t tmpZ("z");
	Vector position;
	for (MSXML2::IXMLDOMNodePtr pChild = positionNode->firstChild;NULL != pChild;pChild = pChild->nextSibling) {
		if(pChild->nodeType == MSXML2::NODE_ELEMENT) {
			if(pChild->nodeName == tmpX) {
				position.x = (FLOAT)atof(pChild->firstChild->text);
			} else if(pChild->nodeName == tmpY) {
				position.y = (FLOAT)atof(pChild->firstChild->text);
			} else if(pChild->nodeName == tmpZ) {
				position.z = (FLOAT)atof(pChild->firstChild->text);
			}
		}
	}
	return position;
}

void ShadowSceneLoader::traceSkyBox(MSXML2::IXMLDOMNodePtr skyBoxNode) {
	_bstr_t tmpCube("xfile");
	std::string skyname;
	
	std::string skyTexture;
	for (MSXML2::IXMLDOMNodePtr pChild = skyBoxNode->firstChild;NULL != pChild;pChild = pChild->nextSibling) {
		if(pChild->nodeType == MSXML2::NODE_ELEMENT) {
			if(pChild->nodeName == tmpCube) {
				skyname = pChild->firstChild->text;
			}
		}
	}

	SkyBox *skybox = (SkyBox *) this->scene->createNode(this->scene->NODE_SKYBOX, *this->scene->getRoot(), false);
	skybox->generateSkyBox(skyname);

}

void ShadowSceneLoader::traceRenderer(MSXML2::IXMLDOMNodePtr rendererNode, Node &node)
{
	/*<renderer>
		<type>0</type>		1 - PARTICLE
		....other renderer parameter....
	 </renderer>*/
	_bstr_t tmpType("type");
	
	int type = 0;

	for (MSXML2::IXMLDOMNodePtr pChild = rendererNode->firstChild;NULL != pChild;pChild = pChild->nextSibling) {
		if(pChild->nodeType == MSXML2::NODE_ELEMENT) {
			if(pChild->nodeName == tmpType) {
				type = atoi(pChild->firstChild->text);
			}
		}
	}
}

Object3d* ShadowSceneLoader::traceEnvironmentObject(MSXML2::IXMLDOMNodePtr envNode) {
	_bstr_t tmpPosition("position");
	_bstr_t tmpRotation("rotation");
	_bstr_t tmpArrivaltime("arrivaltime");
	_bstr_t tmpVariation("timevariation");
	_bstr_t tmpRenderer("renderer");
	_bstr_t tmpXFile("xfile");
	_bstr_t tmpSoftKill("softkill");
	_bstr_t tmpSceneID("id");
	_bstr_t tmpFixedInScene("fixed");


	Vector position;
	Vector rotation;
	float arrivalTime = 0;
	float arrivalVariation = 0;
	float softKill=0;
	MSXML2::IXMLDOMNodePtr rendererChild;
	bool hasRenderer = false;
	std::string xfile = "";
	int iSceneId = 0;
	bool bFixed = false;

	for (MSXML2::IXMLDOMNodePtr pChild = envNode->firstChild;NULL != pChild;pChild = pChild->nextSibling) {
		if(pChild->nodeType == MSXML2::NODE_ELEMENT) {
			if(pChild->nodeName == tmpPosition) {
				position = tracePosition(pChild);
			} else if(pChild->nodeName == tmpRotation) {
				rotation = tracePosition(pChild);
			}else if(pChild->nodeName == tmpArrivaltime) {
				arrivalTime = (FLOAT)atof(pChild->firstChild->text);
			} else if(pChild->nodeName == tmpVariation) {
				arrivalVariation = (FLOAT)atof(pChild->firstChild->text);
			} else if(pChild->nodeName == tmpSceneID) {
				iSceneId = (INT)atoi(pChild->firstChild->text);
			} else if(pChild->nodeName == tmpFixedInScene) {
				bFixed = ((INT)atoi(pChild->firstChild->text)) == 1 ? true : false;
			} else if(pChild->nodeName == tmpRenderer) {
				rendererChild = pChild;
				hasRenderer = true;
			} else if(pChild->nodeName == tmpXFile) {
				xfile = pChild->firstChild->text;
			} else if(pChild->nodeName == tmpSoftKill) {
				softKill = (FLOAT)atof(pChild->firstChild->text);
			}
		}
	}
	rotation = rotation*D3DX_PI/180;

	Object3d* obj = (Object3d*) this->scene->createNode(SoftShadowScene::NODE_OBJECT3D, !hasRenderer);
	
	obj->setPosition(position);
	obj->setRotation(rotation);
	obj->loadMeshFromFile(xfile);
	if(softKill!=0) {
		obj->setSoftKill(true, softKill);
	}
	if(arrivalTime!=0) {
		obj->setStandBy(true);
		this->scene->setTrigger(SoftShadowScene::TRIGGER_UNSETSTANDBY, obj, this->scene->getRandomTime(arrivalTime, arrivalVariation));
	}
	if(iSceneId != 0) {
		obj->iSceneNumber = iSceneId;
		obj->bFixed = bFixed;
		this->scene->sceneObjectList.push_back(obj);
	}
	return obj;
}

SoundNode* ShadowSceneLoader::traceSound(MSXML2::IXMLDOMNodePtr soundNode)
{
	_bstr_t tmpPosition("position");
	_bstr_t tmpArrivaltime("arrivaltime");
	_bstr_t tmpVariation("timevariation");
	_bstr_t tmpSoundfile("soundfile");
	_bstr_t tmpSoundVolume("volume");
	_bstr_t tmpKillAfterPlayed("killafterplayed");
	_bstr_t tmpLoop("loop");
    
	Vector position;
	float arrivalTime = 0;
	float arrivalVariation = 0;
	std::string soundfile;
	float volume = 1;
	bool killAfterPlayed;
	bool loop;
	
	for (MSXML2::IXMLDOMNodePtr pChild = soundNode->firstChild;NULL != pChild;pChild = pChild->nextSibling) {
		if(pChild->nodeType == MSXML2::NODE_ELEMENT) {
			if(pChild->nodeName == tmpPosition) {
				position = tracePosition(pChild);
			} else if(pChild->nodeName == tmpArrivaltime) {
				arrivalTime = (FLOAT)atof(pChild->firstChild->text);
			} else if(pChild->nodeName == tmpVariation) {
				arrivalVariation = (FLOAT)atof(pChild->firstChild->text);
			} else if(pChild->nodeName == tmpSoundfile) {
				soundfile = pChild->firstChild->text;
			} else if(pChild->nodeName == tmpSoundVolume) {
				volume = (FLOAT) atof(pChild->firstChild->text);
			} else if(pChild->nodeName == tmpKillAfterPlayed) {
				killAfterPlayed = ((INT)atoi(pChild->firstChild->text)==1);
			} else if(pChild->nodeName == tmpLoop) {
				loop = ((INT)atoi(pChild->firstChild->text)==1);
			}
		}
	}

	SoundNode* sound;
	sound = (SoundNode*) this->scene->createNode(Scene::NODE_SOUND);
	sound->loadFile(soundfile, loop);
	sound->setVolume(volume);
	sound->setKillSoundNodeAfterPlayed(killAfterPlayed);
	
	if(arrivalTime!=0) {
		sound->setStandBy(true);
		this->scene->setTrigger(SoftShadowScene::TRIGGER_UNSETSTANDBY, sound, this->scene->getRandomTime(arrivalTime, arrivalVariation));
	} else {
		sound->play();
	}

	return sound;
}

void ShadowSceneLoader::traceSoftShadowLight(MSXML2::IXMLDOMNodePtr envNode)
{
	_bstr_t tmpPosition("position");
	_bstr_t tmpLookAt("rotation");
	_bstr_t tmpLength("length");
	_bstr_t tmpEpsilon("epsilon");
	_bstr_t tmpMapSize("mapsize");
	
	Vector position;
	Vector rotation;
	float length = 0;
	float epsilon = 0;
	int mapSize = 128;
	int errorMetric = 0;

	for (MSXML2::IXMLDOMNodePtr pChild = envNode->firstChild;NULL != pChild;pChild = pChild->nextSibling) {
		if(pChild->nodeType == MSXML2::NODE_ELEMENT) {
			if(pChild->nodeName == tmpPosition) {
				position = tracePosition(pChild);
			} else if(pChild->nodeName == tmpLookAt) {
				rotation = tracePosition(pChild);
			} else if(pChild->nodeName == tmpLength) {
				length = (FLOAT)atof(pChild->firstChild->text);
			} else if(pChild->nodeName == tmpEpsilon) {
				epsilon = (FLOAT)atof(pChild->firstChild->text);
			} else if(pChild->nodeName == tmpMapSize) {
				mapSize = (INT)atoi(pChild->firstChild->text);
			} 
		}
	}

	LowMemLight* light = (LowMemLight*) this->scene->createNode(SoftShadowScene::NODE_SOFTSHADOWLOWMEMLIGHT);
	
	rotation = rotation*3.14f/180.0f;

	light->setPosition(position);
	light->setRotation(rotation);
	light->setErrorMetric(errorMetric);
	light->setLightDimension(length);
	light->setEpsilon(epsilon);
	light->setShadowMapSize(mapSize);
	light->setLowResShadowMap(mapSize);

	this->scene->light = light;
}
