#pragma once
#import <MSXML2.dll> named_guids
#include "SoftShadowScene.h"

using namespace MSXML2; 

class Object3d;
class NxActorDesc;
class NxBodyDesc;
class Sprite;
class SoftShadowLight;

#define SPTR boost::shared_ptr
#define WPTR boost::weak_ptr

/**
ShadowSceneLoader class
*/

class ShadowSceneLoader
{
public:
	/**
	Constructor for ShadowSceneLoader object
	*/
	ShadowSceneLoader(void);
	/**
	Destructor for ShadowSceneLoader object
	*/
	~ShadowSceneLoader(void);

	/**
	Load a soft shadow scene
	*/
	bool loadSoftShadowScene(SoftShadowScene &_scene, std::string filename, GameManager *_manager);

	/**
	Trace position of a node
	*/
	Vector tracePosition(MSXML2::IXMLDOMNodePtr positionNode);
	/**
	Trace sun
	*/
	void traceSun(MSXML2::IXMLDOMNodePtr sunNode);
	/**
	Trace sky box
	*/
	void traceSkyBox(MSXML2::IXMLDOMNodePtr skyBoxNode);
	/**
	Trace renderer object
	*/
	void traceRenderer(MSXML2::IXMLDOMNodePtr rendererNode, Node &node);
	/**
	Trace an environment object (Object3d)
	*/
	Object3d* traceEnvironmentObject(MSXML2::IXMLDOMNodePtr envNode);
	/**
	Trace a sprite node
	*/
	Sprite* traceSprite(MSXML2::IXMLDOMNodePtr spriteNode);
	/**
	Trace sound node
	*/
	SoundNode* traceSound(MSXML2::IXMLDOMNodePtr soundNode);
	/**
	Trace SoftShadow light
	*/
	void traceSoftShadowLight(MSXML2::IXMLDOMNodePtr envNode);
protected:
	SoftShadowScene *scene;
	MSXML2::IXMLDOMDocumentPtr sceneDomDocument;
	MSXML2::IXMLDOMElementPtr  sceneDocRoot;
};
