#pragma once
#include "renderer.h"
#include "Object3d.h"

/**
SimpleMeshRenderer class
*/

class SimpleMeshRenderer :
	public Renderer
{
public:
	/**
	Constructor for SimpleMeshRenderer object
	*/
	SimpleMeshRenderer(void);
	/**
	Destructor for SimpleMeshRenderer object
	*/
	virtual ~SimpleMeshRenderer(void);

	/**
	Initialization
	*/
	virtual void init();
	/**
	Render node
	*/
	virtual void render();

	/**
	Enable/Disable Alpha Blending
	*/
	void enableAlphaBlending(bool _alpha);

	virtual void OnLostDevice( void* pUserContext );
	virtual void OnDestroyDevice( void* pUserContext );
	virtual HRESULT OnResetDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext );
	
	virtual bool isLowerThan(Renderer* renderer);

protected:

	/**
	Render default pass
	*/
	void defaultPass();
	/**
	Render depth pass
	*/
	void depthPass();
	/**
	Render shadow map pass
	*/
	void shadowMapPass();
	/**
	Render pass which sums up values from the depth maps
	*/
	void addSMPass();

	bool alphaBlending;
	ID3DXEffect* depthMapEffect;
	ID3DXEffect* simpleMeshEffect;
};
