#include "dxstdafx.h"
#include ".\SkyBoxRenderer.h"
#include "GameManager.h"
#include "Camera.h"

SkyBoxRenderer::SkyBoxRenderer(void) : Renderer()
{
	this->setRenderPriority(0);
	this->rendererType |= Renderer::RENDERER_SKYBOX;
}

SkyBoxRenderer::~SkyBoxRenderer(void)
{
}

void SkyBoxRenderer::init() {
	this->shadowEffect = this->myScene->manager->getEffect(GameManager::EFFECT_SHADOWMAP);
	if(!this->shadowEffect) {
		this->shadowEffect = this->myScene->manager->loadEffect(GameManager::EFFECT_SHADOWMAP, L"shaders/shadowMap.fx");
	}
}

void SkyBoxRenderer::render() 
{

	this->device->SetTransform(D3DTS_WORLD, &this->myScene->getIdentityMatrix());
	this->device->SetTransform(D3DTS_VIEW, &this->myScene->getIdentityMatrix());
	this->device->SetTransform(D3DTS_PROJECTION, &this->myScene->getIdentityMatrix());
	SPTR<Node> node = this->myNode.lock();
	Object3d *obj = (Object3d *) (node.get());
	switch(this->myScene->getActivePassId()) {
		case Scene::PASS_NORMAL:
			this->device->BeginScene();
			if(obj!=NULL && obj->isModelLoaded()) {
				this->device->SetTransform(D3DTS_WORLD, obj->getWorldMatrix());
				this->device->SetTransform(D3DTS_VIEW, this->myScene->getActiveCamera()->getViewMatrix());
				this->device->SetTransform(D3DTS_PROJECTION, (this->myScene->getActiveCamera()->getProjectionMatrix()));
				this->device->SetRenderState( D3DRS_CULLMODE, D3DCULL_CW );
				this->device->SetRenderState( D3DRS_LIGHTING, FALSE );
				this->device->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID);
				
				// bilinear texture filtering:
				this->device->SetSamplerState( 0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR );
				this->device->SetSamplerState( 0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR );
				this->device->SetSamplerState( 0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR );

				// Disable writing to the z buffer
				// achtung: im moment alles standard (true, lessequal, true)
				this->device->SetRenderState( D3DRS_ZENABLE, FALSE );
				this->device->SetRenderState( D3DRS_ZWRITEENABLE, FALSE );

				// Clamp texture co-ordinates
				this->device->SetSamplerState( 0, D3DSAMP_ADDRESSU,  D3DTADDRESS_CLAMP );
				this->device->SetSamplerState( 0, D3DSAMP_ADDRESSV,  D3DTADDRESS_CLAMP );

				for(UINT i = 0; i < obj->Materials.size(); i++) {
					
					if(myScene->useTextures) {
						this->device->SetTexture(0, obj->Textures[i]);
					} else {
						obj->Materials[i].Ambient.r = 0;
						this->device->SetTexture(0, 0);
					}
					this->device->SetMaterial( &obj->Materials[i] );
					
					(*obj->getMesh())->DrawSubset(i);

				}
				
				// Wrap texture co-ordinates again
				this->device->SetSamplerState( 0, D3DSAMP_ADDRESSU,  D3DTADDRESS_WRAP );
				this->device->SetSamplerState( 0, D3DSAMP_ADDRESSV,  D3DTADDRESS_WRAP );

				// Enable writing to the z buffer again
				this->device->SetRenderState( D3DRS_ZENABLE, TRUE );
				this->device->SetRenderState( D3DRS_ZWRITEENABLE, TRUE );
				DXUTGetD3DDevice()->Clear(0, NULL, D3DCLEAR_ZBUFFER, D3DCOLOR_ARGB(0, 0, 0, 0), 1.0f, 0);
			}
			this->device->SetTransform(D3DTS_WORLD, &this->myScene->getIdentityMatrix());
			this->device->SetTransform(D3DTS_VIEW, &this->myScene->getIdentityMatrix());
			this->device->SetTransform(D3DTS_PROJECTION, &this->myScene->getIdentityMatrix());
			this->device->EndScene();
			break;
		case Scene::PASS_DEPTH:
		case Scene::PASS_SOFTSHADOWMAP:
		case Scene::PASS_REPROJECTION:
			HRESULT hr;
			UINT cPass;


			V( shadowEffect->SetTechnique("ClearDepthMap") );
			V( shadowEffect->Begin( &cPass, 0 ) );
			V( shadowEffect->BeginPass( 0 ) );
			V( this->device->BeginScene() );

			V( this->myScene->getQuadMesh()->DrawSubset(0) );

			V( this->device->EndScene() );
			V( shadowEffect->EndPass() );
			V( shadowEffect->End() );
			
			DXUTGetD3DDevice()->Clear(0, NULL, D3DCLEAR_ZBUFFER, D3DCOLOR_ARGB(255, 255, 255, 255), 1.0f, 0);

			break;
	}
	this->device->SetTransform(D3DTS_WORLD, &this->myScene->getIdentityMatrix());
	this->device->SetTransform(D3DTS_VIEW, &this->myScene->getIdentityMatrix());
	this->device->SetTransform(D3DTS_PROJECTION, &this->myScene->getIdentityMatrix());
}

void SkyBoxRenderer::OnLostDevice( void* pUserContext )
{
	this->myScene->manager->printToConsole(" OnLostDevice SkyBoxRenderer");
}

void SkyBoxRenderer::OnDestroyDevice( void* pUserContext )
{
	this->myScene->manager->printToConsole(" OnDestroyDevice SkyBoxRenderer");
}

HRESULT SkyBoxRenderer::OnResetDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
	this->device = pd3dDevice;
	return S_OK;
}

bool SkyBoxRenderer::isLowerThan(Renderer* renderer)
{
	return this->getRenderPriority() < renderer->getRenderPriority();
}
