#include "dxstdafx.h"
#include "SoftShadowLightRenderer.h"
#include "Scene.h"
#include "LowMemLight.h"
#include "GameManager.h"

SoftShadowLightRenderer::SoftShadowLightRenderer(void)
{
	this->setRenderPriority(1000);
	this->rendererType |= Renderer::RENDERER_SOFTSHADOWLIGHT;
}

SoftShadowLightRenderer::~SoftShadowLightRenderer(void)
{
	SAFE_RELEASE(lightLine);
}

bool SoftShadowLightRenderer::isLowerThan(Renderer* renderer)
{
	return this->getRenderPriority()<renderer->getRenderPriority();
}

void SoftShadowLightRenderer::OnLostDevice( void* pUserContext )
{
}

void SoftShadowLightRenderer::OnDestroyDevice( void* pUserContext )
{
}

HRESULT SoftShadowLightRenderer::OnResetDevice( IDirect3DDevice9* pd3dDevice, const D3DSURFACE_DESC* pBackBufferSurfaceDesc, void* pUserContext )
{
	return S_OK;
}

void SoftShadowLightRenderer::init()
{
	this->device->CreateVertexBuffer( 4*sizeof(CUSTOMVERTEX), D3DUSAGE_WRITEONLY, vertexFVF, D3DPOOL_DEFAULT, &lightLine, NULL );
}

void SoftShadowLightRenderer::render()
{
	HRESULT hr;

	if(this->myScene->getActivePassId()!=Scene::PASS_NORMAL)
		return;

	//Rendere LightSource (Linie)
	SPTR<Node> node = this->myNode.lock();	
	LowMemLight* light = (LowMemLight *) (node.get());
	Vector color(0,1,0);

	//Set the values for each vertex. 
	CUSTOMVERTEX * vList;
	lightLine->Lock( 0, 0, (VOID**)&vList, 0 );

	//XAXIS
	vList[0].x = -light->getLightDimension()/2;
	vList[0].y = 0;
	vList[0].z = 0;
	vList[0].emissive = D3DCOLOR_ARGB(0, (int)(255*color.x),(int)(255*color.y), (int)(255*color.z));
	vList[1].x = light->getLightDimension()/2;
	vList[1].y = 0;
	vList[1].z = 0;
	vList[1].emissive = D3DCOLOR_ARGB(0, (int)(255*color.x),(int)(255*color.y), (int)(255*color.z));
	vList[2].x = 0;
	vList[2].y = 0;
	vList[2].z = 0;
	vList[2].emissive = D3DCOLOR_ARGB(0, (int)(255*color.x),(int)(255*color.y), (int)(255*color.z));
	vList[3].x = 0;
	vList[3].y = 0;
	vList[3].z = light->getLightDimension()/4;
	vList[3].emissive = D3DCOLOR_ARGB(0, (int)(255*color.x),(int)(255*color.y), (int)(255*color.z));

	lightLine->Unlock();

	V( this->device->SetRenderState( D3DRS_LIGHTING, FALSE ) );
	V( this->device->SetRenderState(D3DRS_FILLMODE, D3DFILL_WIREFRAME) );

	//Render Light
	D3DXMATRIX identity;
	D3DXMatrixIdentity(&identity);

	D3DXMATRIX invLightMat;
	D3DXMatrixInverse(&invLightMat, NULL, light->getWorldMatrix());
	this->device->SetTransform(D3DTS_PROJECTION, this->myScene->getActiveRenderPass()->getProjectionMatrix() );
	this->device->SetTransform( D3DTS_VIEW, &identity );
	this->device->SetTransform( D3DTS_WORLD, &this->myScene->getActiveRenderPass()->getWorldViewMatrix(&invLightMat) );
		
	V( this->device->BeginScene() );
	V( this->device->SetFVF(vertexFVF) );
	V( this->device->SetStreamSource(0, lightLine, 0, sizeof(CUSTOMVERTEX)) );
	V( this->device->DrawPrimitive(D3DPT_LINELIST, 0 , 2) );
	V( this->device->EndScene() );

	this->device->SetTransform(D3DTS_PROJECTION, &identity );
	this->device->SetTransform( D3DTS_VIEW, &identity );
	this->device->SetTransform( D3DTS_WORLD, &identity );

	V( this->device->SetRenderState( D3DRS_LIGHTING, TRUE ) );
	V( this->device->SetRenderState(D3DRS_FILLMODE, D3DFILL_SOLID) );
}
