#include "dxstdafx.h"
#include "SoftShadowScene.h"
#include "SkyBox.h"
#include "SoundNode.h"
#include "Box.h"
#include <cstdlib>
#include <ctime>
#include <typeinfo>
#include <string>
#include "ShadowSceneLoader.h"
#include "LowMemLight.h"

#define M_PI 3.1415

SoftShadowScene::SoftShadowScene(void) : Scene()
{
	srand( (unsigned)time( NULL ) );
	this->skybox = 0;
	this->bgSoundChannel = -1;
	
	this->bGUIVisible = true;

	//set min/max values for sliders:
	MIN_EPSILON = 0.01f;
	MAX_EPSILON = 0.3f;

	MIN_ABS = 0.005f;
	MAX_ABS = 0.05f;

	MIN_BIAS = 1.0f/5000.0f;
	MAX_BIAS = 1.0f/500.0f;

	MIN_LIGHTDIM = 1.0f;
	MAX_LIGHTDIM = 20.0f;

	MIN_SAMPLINGDIST = 0.001f;
	MAX_SAMPLINGDIST = 0.02f;

	bFirstGUIRendering = true;

	iCurrentSceneId = 1;
	this->light = NULL;
}

SoftShadowScene::~SoftShadowScene(void)
{
}

void SoftShadowScene::clearScene()
{
	this->skybox = 0;
	this->bgSoundChannel = -1;
	this->firstFrame = true;
	
	this->cleanUpScene();

	materialXMLVector.clear();
	materialPhysicVector.clear();
	
	Scene::clearScene();
}

void SoftShadowScene::initScene(GameManager &_manager) {
	Scene::initScene(_manager);

	//Setup Camera
	this->activeCamera->setLookAtTarget(150, 50, 150);
	this->camDesRadius = 30;
	this->camRadius = 30;
	this->camDesYaw = -D3DX_PI/3;
	this->camYaw = -D3DX_PI/3;
	this->camDesPitch = D3DX_PI/9;
	this->camPitch = D3DX_PI/9;
	this->camFactor = 120;
	this->maxPitch = D3DX_PI/1.5;
	this->mousePressed = false;
	this->rightMousePressed = false;
	this->activeCamera->setPosition(100, -200, 100);
	this->orthoDimension = 25;

	ShadowSceneLoader loader;
	if(!loader.loadSoftShadowScene(*this, "./media/challenges/scene.xml", this->manager)) {
		this->manager->printToConsole("Loading failed!");	
	}

	//Initialize rest
	this->lookAtTarget = Vector(150,10,150);
	this->desiredLookAtTarget = Vector(150,10,150);
	this->camRadius = 80.0f;
	this->camDesRadius = 80.0f;
}

void SoftShadowScene::renderScene(float fElapsedTime) 
{
	if(this->isVisible()) {
		if(this->device==0) {
			this->device = DXUTGetD3DDevice();
			this->activeCamera->setupProjectionMatrix();
			this->setLight(Vector(1, 1, 1));
		}

		if(this->firstFrame) {
			this->playBackgroundSound();
			this->setupGUI();
		}

		this->takeTime();
		
		//set current scene objects visible
		if(keyDown['1']) {
			iCurrentSceneId = 1;
		} else if (keyDown['2']) {
			iCurrentSceneId = 2;
		} else if (keyDown['3']) {
			iCurrentSceneId = 3;
		} else if (keyDown['4']) {
			iCurrentSceneId = 4;
		} else if (keyDown['5']) {
			iCurrentSceneId = 5;
		} else if (keyDown['6']) {
			iCurrentSceneId = 6;
		}  

		Object3d* currentObject = 0;
		float speed = 5.0f;

		for(UINT i=0;i<this->sceneObjectList.size();i++) 
		{		
			currentObject = this->sceneObjectList.at(i);

			if(currentObject->iSceneNumber == iCurrentSceneId) {
				currentObject->setVisible(true);
				if(!this->altDown && !currentObject->bFixed) {
					Vector dPos = currentObject->getPosition();
					Vector dRot;
					dRot.x = currentObject->getRotationX();
					dRot.y = currentObject->getRotationY();
					dRot.z = currentObject->getRotationZ();
					if(this->keyDown['S'])
						dPos.x+=speed*dt;
					if(this->keyDown['W'])
						dPos.x-=speed*dt;
					if(this->keyDown['E'])
						dPos.y+=speed*dt;
					if(this->keyDown['C'])
						dPos.y-=speed*dt;
					if(this->keyDown['D'])
						dPos.z+=speed*dt;
					if(this->keyDown['A'])
						dPos.z-=speed*dt;
					if(this->keyDown['I'])
						dRot.x+=speed*dt;
					if(this->keyDown['K'])
						dRot.x-=speed*dt;
					if(this->keyDown['J'])
						dRot.y+=speed*dt;
					if(this->keyDown['L'])
						dRot.y-=speed*dt;
					if(this->keyDown['U'])
						dRot.z+=speed*dt;
					if(this->keyDown['M'])
						dRot.z-=speed*dt;

					currentObject->setPosition(dPos);
					currentObject->setRotation(dRot);
				}
			} else {
				currentObject->setVisible(false);
			}
		}

		//Update LightPosition
		if(this->light!=NULL && this->altDown) {
			Vector dPos = light->getPosition();
			Vector dRot;
			dRot.x = light->getRotationX();
			dRot.y = light->getRotationY();
			dRot.z = light->getRotationZ();
			if(this->keyDown['S'])
				dPos.x+=speed*dt;
			if(this->keyDown['W'])
				dPos.x-=speed*dt;
			if(this->keyDown['E'])
				dPos.y+=speed*dt;
			if(this->keyDown['C'])
				dPos.y-=speed*dt;
			if(this->keyDown['D'])
				dPos.z+=speed*dt;
			if(this->keyDown['A'])
				dPos.z-=speed*dt;
			if(this->keyDown['I'])
				dRot.z-=speed*dt;
			if(this->keyDown['K'])
				dRot.z+=speed*dt;
			if(this->keyDown['J'])
				dRot.y+=speed*dt;
			if(this->keyDown['L'])
				dRot.y-=speed*dt;
			if(this->keyDown['U'])
				dRot.x+=speed*dt;
			if(this->keyDown['M'])
				dRot.x-=speed*dt;

			light->setPosition(dPos);
			light->setRotation(dRot);
		}

		//Clear SharedResources
		this->clearSharedResources();
				
		//Fading Stuff
        this->doFade();

		//CAMERA UPDATE
		this->updateCamera(fElapsedTime);

		//Trigger updates
		this->updateTrigger();
		//Update Nodes
		this->updateNodes();
		//Calculate Transformations
		this->calculateTransformations();
				
		//startRenderPasses
		this->startRenderPasses();

		//Render GUI:
		if(this->bGUIVisible)
			this->renderGUI(fElapsedTime);

		//Cleanup Memory
		this->cleanUpScene();

		this->firstFrame = false;
	}
}

void SoftShadowScene::cleanUpScene()
{
	std::list<SPTR<Node> >::iterator it;
	std::list<SPTR<Node> >::iterator nit;
	std::vector<SPTR<Renderer> >::iterator rit;
	Node* node;
	for(it=this->zombieNodeList.begin();it!=this->zombieNodeList.end();it++) {
		node = (*it).get();
		node->getFather()->removeChild(*it);

		//Delete Renderer
		if(node->hasRenderer())
		{
			Renderer *r = node->getRenderer().get();
			for(rit=this->rendererList.begin();rit!=this->rendererList.end();rit++) {
				if((*rit).get() == r)
				{
					this->rendererList.erase(rit);
					break;
				}
			}
		}

		this->deleteNodeInList(node, this->nodeList);
		this->deleteNodeInList(node, this->soundNodeList);
	}
	this->zombieNodeList.clear();
}

Node* SoftShadowScene::createNode(int type)
{
	return this->createNode(type, this->root, true, false);
}

Node* SoftShadowScene::createNode(int type, Node &father)
{
	return this->createNode(type, father, true, false);
}

Node* SoftShadowScene::createNode(int type, bool addDefaultRenderer)
{
	return this->createNode(type, this->root, addDefaultRenderer, false);
}

Node* SoftShadowScene::createNode(int type, Node &father, bool addDefaultRenderer) {
	return this->createNode(type, father, addDefaultRenderer, false);
}

Node* SoftShadowScene::createNode(int type, Node &father, bool addDefaultRenderer, bool isReference)
{
	switch(type)  {
		case NODE_SKYBOX:
			{
				SPTR<Node> node(new SkyBox);
				father.addChild(node);
				this->nodeList.push_back(node);
				node->setScene(*this);
				Node *temp = node.get();
				this->skybox = (SkyBox *) temp;
				return temp;
			}
			break;
		
		default:
			return Scene::createNode(type, father, addDefaultRenderer, isReference);
			break;
	}
}

float SoftShadowScene::getRandomTime(float timeBase, float timeVariation)
{
	return timeBase + ((((float) rand())/RAND_MAX)-0.5f)*timeVariation*2;
}

void SoftShadowScene::executeTrigger(SPTR<Trigger> trigger)
{
	Scene::executeTrigger(trigger);

	Trigger *t = trigger.get();
}


void SoftShadowScene::connectMaterialIds(int physicId, int xmlId)
{
	this->materialXMLVector.push_back(xmlId);
	this->materialPhysicVector.push_back(physicId);
}

int SoftShadowScene::getPhysicMaterialId(int xmlId)
{
	for(UINT i=0;i<this->materialXMLVector.size();i++) {
		if(this->materialXMLVector.at(i) == xmlId) {
			return this->materialPhysicVector.at(i);
		}
	}
	return 0;
}

void SoftShadowScene::updateCamera(float dt)
{
	if(ActionInMenu())
		return;
	
	//Check leftmousebutton
	if(!this->mousePressed && this->mouseButton[Scene::MOUSE_LEFT]) {
		this->mousePressed = true;
		this->oldMousePos[0] = (float)this->mousePos[0];
		this->oldMousePos[1] = (float)this->mousePos[1];
	} else if(this->mousePressed && !this->mouseButton[Scene::MOUSE_LEFT]) {
		this->mousePressed = false;
	} else if(this->mousePressed) {	//Mouse dragging
		this->camDesYaw += -(this->oldMousePos[0]-this->mousePos[0])/(this->manager->screenWidth/2)*D3DX_PI;
		this->camDesPitch += -(this->oldMousePos[1]-this->mousePos[1])/this->manager->screenHeight*this->maxPitch;
		this->oldMousePos[0] = (float)this->mousePos[0];
		this->oldMousePos[1] = (float)this->mousePos[1];
	}
	//check rightmousebutton
	if(!this->rightMousePressed && this->mouseButton[Scene::MOUSE_RIGHT]) {
		this->rightMousePressed = true;
		this->oldZoomMousePos = (float) this->mousePos[1];
	} else if(this->rightMousePressed && !this->mouseButton[Scene::MOUSE_RIGHT]) {
		this->rightMousePressed = false;
	} else if(this->rightMousePressed) {	//Mouse dragging
		this->camDesRadius -= (this->oldZoomMousePos - this->mousePos[1])*5*dt;
		this->camDesRadius = max(this->camDesRadius, 7.0f);
		this->oldZoomMousePos = (float) this->mousePos[1];
	}
	
	this->camRadius	+= (this->camDesRadius - this->camRadius)*min(this->camFactor*dt, 1.0f);
	this->camYaw	+= (this->camDesYaw - this->camYaw)*min(this->camFactor*dt, 1.0f);
	this->camPitch	+= (this->camDesPitch - this->camPitch)*min(this->camFactor*dt, 1.0f);
	this->lookAtTarget = this->lookAtTarget + (this->desiredLookAtTarget - this->lookAtTarget)*min(this->camFactor*dt, 1.0f);

	D3DXMATRIX rotMat;
	D3DXMATRIX transMat;
	Vector originPos(0, 0, -this->camRadius);
	D3DXMatrixRotationYawPitchRoll( &rotMat, this->camYaw, this->camPitch, 0 );
	D3DXMatrixTranslation(&transMat, this->lookAtTarget.x, this->lookAtTarget.y, this->lookAtTarget.z);
	D3DXMatrixMultiply(&rotMat, &rotMat, &transMat);
	D3DXVec4Transform(&this->camPos, &originPos, &rotMat);
	this->activeCamera->setPosition(this->camPos);
	this->activeCamera->setLookAtTarget(this->lookAtTarget);

	this->skybox->getBoxObject()->setPosition(camPos.x, camPos.y, camPos.z);
}

bool SoftShadowScene::ActionInMenu() {
	if(!bGUIVisible)
		return false;

	if(GUI->GetEditBox( EDB_BACKGROUND )->GetVisible()) {
		if(this->mousePos[0] >= GUI->GetEditBox( EDB_BACKGROUND )->m_x && 
			this->mousePos[0] <= GUI->GetEditBox( EDB_BACKGROUND )->m_x + GUI->GetEditBox( EDB_BACKGROUND )->m_width && 
			this->mousePos[1] >= GUI->GetEditBox( EDB_BACKGROUND )->m_y && 
			this->mousePos[1] <= GUI->GetEditBox( EDB_BACKGROUND )->m_y + GUI->GetEditBox( EDB_BACKGROUND )->m_height)
		{
			return true;
		}
	}

	return false;
}

void SoftShadowScene::setupGUI()
{
	//calculate size factors:
	float factorX = ((float)this->manager->screenWidth)/((float)100);
	float factorY = ((float)this->manager->screenHeight)/((float)100);

	//set GUI visibility to true:
	this->GUI->SetVisible(true);

	//set font size:
	int fontsize = 20 * manager->screenWidth/1024;
	this->GUI->SetFont( 0, L"Arial", fontsize, FW_NORMAL );
	this->GUI->SetFont( 1, L"Arial", (int)(fontsize*0.75f), FW_NORMAL );
	this->GUI->SetFont( 2, L"Arial", (int)(fontsize*0.6f), FW_NORMAL );
	this->GUI->SetFont( 3, L"Arial", fontsize, FW_BOLD );

	//add edit boxes (i use disabled edit boxes as background)
	this->GUI->AddEditBox( EDB_BACKGROUND, L"", 20, 20, 300, 300 );
	
	// add buttons:
	this->GUI->AddButton( BUT_EXIT, L"Exit", 10, 10, 10, 10 );
	
	// add combobox
    CDXUTComboBox *pCombo1;
    this->GUI->AddComboBox( CMB_ZERO, 0, 0, 200, 24, 0, false, &pCombo1 );
	CDXUTComboBox *pCombo2;
    this->GUI->AddComboBox( CMB_ONE, 0, 0, 200, 24, 0, false, &pCombo2 );
	CDXUTComboBox *pCombo3;
    this->GUI->AddComboBox( CMB_TWO, 0, 0, 200, 24, 0, false, &pCombo3 );
	
	// add Slider
    this->GUI->AddSlider( SLD_ZERO, 0, 0, 10, 10, 0, 100, 50, false );
    this->GUI->AddSlider( SLD_ONE, 0, 0, 10, 10, 0, 100, 50, false );
	this->GUI->AddSlider( SLD_TWO, 0, 0, 10, 10, 0, 100, 50, false );
	this->GUI->AddSlider( SLD_THREE, 0, 0, 10, 10, 0, 100, 50, false );
	this->GUI->AddSlider( SLD_FOUR, 0, 0, 10, 10, 0, 100, 50, false );
    
	// add checkboxes:
	this->GUI->AddCheckBox( CHB_ZERO, L"Calc. from view point", 0, 0, 10, 10, false, false );
	this->GUI->AddCheckBox( CHB_ONE, L"Use Textures", 0, 0, 10, 10, false, false );
	this->GUI->AddCheckBox( CHB_TWO, L"Show debug info", 0, 0, 10, 10, false, false );
	
	//add statics:
	this->GUI->AddStatic( STA_ZERO, L"Press F1 to toggle GUI on/off", 0, 0, 200, 30 );
	this->GUI->AddStatic( STA_ONE, L"FPS: 0", 0, 0, 200, 30 );
	this->GUI->AddStatic( STA_TWO, L"Kernel size:", 0, 0, 200, 30 );
	this->GUI->AddStatic( STA_THREE, L"Epsilon: 0", 0, 0, 200, 30 );
	this->GUI->AddStatic( STA_FOUR, L"Max. depth diff.: 0", 0, 0, 200, 30 );
	this->GUI->AddStatic( STA_FIVE, L"Number of depth maps: 0", 0, 0, 200, 30 );
	this->GUI->AddStatic( STA_SIX, L"Shadow map bias: 0", 0, 0, 200, 30 );
	this->GUI->AddStatic( STA_SEVEN, L"Filter mode:", 0, 0, 200, 30 );
	this->GUI->AddStatic( STA_EIGHT, L"Light size: 0", 0, 0, 200, 30 );
	this->GUI->AddStatic( STA_NINE, L"Sampling distance: 0", 0, 0, 200, 30 );
	this->GUI->AddStatic( STA_TEN, L"Algorithm:", 0, 0, 200, 30 );
	
	//edit edit boxes:
	this->GUI->GetEditBox( EDB_BACKGROUND )->SetLocation((int)(1*factorX), (int)(1*factorY));
	this->GUI->GetEditBox( EDB_BACKGROUND )->SetSize((int)(25*factorX), (int)(92*factorY));
	this->GUI->GetEditBox( EDB_BACKGROUND )->SetEnabled(false);
	this->GUI->GetEditBox( EDB_BACKGROUND )->SetBorderWidth(5);
    this->GUI->GetControl( EDB_BACKGROUND )->GetElement( 0 )->TextureColor.Init( D3DCOLOR_ARGB( 38, 0, 155, 255 ) );  // Transparent center

	//edit buttons:
	this->GUI->GetButton(BUT_EXIT)->SetLocation((int)(91*factorX), (int)(95*factorY));
	this->GUI->GetButton(BUT_EXIT)->SetSize((int)(8*factorX), (int)(4*factorY));
	this->GUI->GetButton(BUT_EXIT)->GetElement(0)->iFont = 1;
	this->GUI->GetButton(BUT_EXIT)->GetElement(1)->iFont = 1;

	//edit comboboxes:
	this->GUI->GetComboBox(CMB_ZERO)->SetDropHeight((int)(8*factorY));
	this->GUI->GetComboBox(CMB_ZERO)->SetLocation((int)(3*factorX), (int)(61*factorY));
	this->GUI->GetComboBox(CMB_ZERO)->SetSize((int)(21*factorX), (int)(5*factorY));
	this->GUI->GetComboBox(CMB_ZERO)->GetElement(0)->iFont = 0;
	this->GUI->GetComboBox(CMB_ZERO)->GetElement(1)->iFont = 0;
	this->GUI->GetComboBox(CMB_ZERO)->GetElement(2)->iFont = 0;
	this->GUI->GetComboBox(CMB_ZERO)->GetElement(3)->iFont = 0;
	this->GUI->GetComboBox(CMB_ZERO)->GetElement(0)->TextureColor.Init( D3DCOLOR_ARGB( 91, 255, 0, 0 ) );
	this->GUI->GetComboBox(CMB_ZERO)->GetElement(1)->TextureColor.Init( D3DCOLOR_ARGB( 111, 255, 0, 0 ), D3DCOLOR_ARGB( 128, 128, 128, 128 ));
	this->GUI->GetComboBox(CMB_ZERO)->GetElement(2)->TextureColor.Init( D3DCOLOR_ARGB( 211, 255, 0, 0 ) );
	this->GUI->GetComboBox(CMB_ZERO)->GetElement(3)->TextureColor.Init( D3DCOLOR_ARGB( 111, 255, 0, 0 ) );
	this->GUI->GetComboBox(CMB_ZERO)->AddItem( L"3x3", (LPVOID)0);
	this->GUI->GetComboBox(CMB_ZERO)->AddItem( L"5x5", (LPVOID)1);
	this->GUI->GetComboBox(CMB_ZERO)->AddItem( L"7x7", (LPVOID)2);
	this->GUI->GetComboBox(CMB_ZERO)->AddItem( L"9x9", (LPVOID)3);
	this->GUI->GetComboBox(CMB_ZERO)->SetVisible(true);
	this->GUI->GetComboBox(CMB_ZERO)->SetSelectedByIndex(1);

	this->GUI->GetComboBox(CMB_ONE)->SetDropHeight((int)(12*factorY));
	this->GUI->GetComboBox(CMB_ONE)->SetLocation((int)(3*factorX), (int)(52*factorY));
	this->GUI->GetComboBox(CMB_ONE)->SetSize((int)(21*factorX), (int)(5*factorY));
	this->GUI->GetComboBox(CMB_ONE)->GetElement(0)->iFont = 0;
	this->GUI->GetComboBox(CMB_ONE)->GetElement(1)->iFont = 0;
	this->GUI->GetComboBox(CMB_ONE)->GetElement(2)->iFont = 0;
	this->GUI->GetComboBox(CMB_ONE)->GetElement(3)->iFont = 0;
	this->GUI->GetComboBox(CMB_ONE)->GetElement(0)->TextureColor.Init( D3DCOLOR_ARGB( 91, 255, 0, 0 ) );
	this->GUI->GetComboBox(CMB_ONE)->GetElement(1)->TextureColor.Init( D3DCOLOR_ARGB( 111, 255, 0, 0 ), D3DCOLOR_ARGB( 128, 128, 128, 128 ));
	this->GUI->GetComboBox(CMB_ONE)->GetElement(2)->TextureColor.Init( D3DCOLOR_ARGB( 211, 255, 0, 0 ) );
	this->GUI->GetComboBox(CMB_ONE)->GetElement(3)->TextureColor.Init( D3DCOLOR_ARGB( 111, 255, 0, 0 ) );
	this->GUI->GetComboBox(CMB_ONE)->AddItem( L"No filter", (LPVOID)0);
	this->GUI->GetComboBox(CMB_ONE)->AddItem( L"PCF filter", (LPVOID)1);
	this->GUI->GetComboBox(CMB_ONE)->AddItem( L"PCF filter 1D", (LPVOID)2);
	this->GUI->GetComboBox(CMB_ONE)->SetSelectedByIndex(this->filterMode);
	this->GUI->GetComboBox(CMB_ONE)->SetVisible(true);

	this->GUI->GetComboBox(CMB_TWO)->SetDropHeight((int)(8*factorY));
	this->GUI->GetComboBox(CMB_TWO)->SetLocation((int)(3*factorX), (int)(77*factorY));
	this->GUI->GetComboBox(CMB_TWO)->SetSize((int)(21*factorX), (int)(5*factorY));
	this->GUI->GetComboBox(CMB_TWO)->GetElement(0)->iFont = 0;
	this->GUI->GetComboBox(CMB_TWO)->GetElement(1)->iFont = 0;
	this->GUI->GetComboBox(CMB_TWO)->GetElement(2)->iFont = 0;
	this->GUI->GetComboBox(CMB_TWO)->GetElement(3)->iFont = 0;
	this->GUI->GetComboBox(CMB_TWO)->GetElement(0)->TextureColor.Init( D3DCOLOR_ARGB( 91, 255, 0, 0 ) );
	this->GUI->GetComboBox(CMB_TWO)->GetElement(1)->TextureColor.Init( D3DCOLOR_ARGB( 111, 255, 0, 0 ), D3DCOLOR_ARGB( 128, 128, 128, 128 ));
	this->GUI->GetComboBox(CMB_TWO)->GetElement(2)->TextureColor.Init( D3DCOLOR_ARGB( 211, 255, 0, 0 ) );
	this->GUI->GetComboBox(CMB_TWO)->GetElement(3)->TextureColor.Init( D3DCOLOR_ARGB( 111, 255, 0, 0 ) );
	this->GUI->GetComboBox(CMB_TWO)->AddItem( L"Our algorithm", (LPVOID)0);
	this->GUI->GetComboBox(CMB_TWO)->AddItem( L"Fixed 128 maps", (LPVOID)1);
	this->GUI->GetComboBox(CMB_TWO)->AddItem( L"Single map, PCF", (LPVOID)2);
	this->GUI->GetComboBox(CMB_TWO)->SetSelectedByIndex(0);
	this->GUI->GetComboBox(CMB_TWO)->SetVisible(true);

	//edit sliders:
	this->GUI->GetSlider(SLD_ZERO)->SetLocation((int)(4*factorX), (int)(26*factorY));
	this->GUI->GetSlider(SLD_ZERO)->SetSize((int)(19*factorX), (int)(3*factorY));
	this->GUI->GetSlider(SLD_ZERO)->GetElement(1)->TextureColor.Init( D3DCOLOR_ARGB( 200, 255, 0, 0 ) );
	this->GUI->GetSlider(SLD_ZERO)->SetVisible(true);
	
	this->GUI->GetSlider(SLD_ONE)->SetLocation((int)(4*factorX), (int)(33*factorY));
	this->GUI->GetSlider(SLD_ONE)->SetSize((int)(19*factorX), (int)(3*factorY));
	this->GUI->GetSlider(SLD_ONE)->GetElement(1)->TextureColor.Init( D3DCOLOR_ARGB( 200, 255, 0, 0 ) );
	this->GUI->GetSlider(SLD_ONE)->SetVisible(true);
	
	this->GUI->GetSlider(SLD_TWO)->SetLocation((int)(4*factorX), (int)(40*factorY));
	this->GUI->GetSlider(SLD_TWO)->SetSize((int)(19*factorX), (int)(3*factorY));
	this->GUI->GetSlider(SLD_TWO)->GetElement(1)->TextureColor.Init( D3DCOLOR_ARGB( 200, 255, 0, 0 ) );
	this->GUI->GetSlider(SLD_TWO)->SetVisible(true);

	this->GUI->GetSlider(SLD_THREE)->SetLocation((int)(4*factorX), (int)(70*factorY));
	this->GUI->GetSlider(SLD_THREE)->SetSize((int)(19*factorX), (int)(3*factorY));
	this->GUI->GetSlider(SLD_THREE)->GetElement(1)->TextureColor.Init( D3DCOLOR_ARGB( 200, 255, 0, 0 ) );
	this->GUI->GetSlider(SLD_THREE)->SetVisible(true);

	this->GUI->GetSlider(SLD_FOUR)->SetLocation((int)(4*factorX), (int)(19*factorY));
	this->GUI->GetSlider(SLD_FOUR)->SetSize((int)(19*factorX), (int)(3*factorY));
	this->GUI->GetSlider(SLD_FOUR)->GetElement(1)->TextureColor.Init( D3DCOLOR_ARGB( 200, 255, 0, 0 ) );
	this->GUI->GetSlider(SLD_FOUR)->SetVisible(true);
	
	//edit check boxes:
	this->GUI->GetCheckBox(CHB_ZERO)->SetLocation((int)(3*factorX), (int)(44*factorY));
	this->GUI->GetCheckBox(CHB_ZERO)->SetSize((int)(21*factorX), (int)(3*factorY));
	this->GUI->GetCheckBox(CHB_ZERO)->GetElement(0)->iFont = 0;
	this->GUI->GetCheckBox(CHB_ZERO)->GetElement(1)->iFont = 0;
	this->GUI->GetCheckBox(CHB_ZERO)->SetVisible(true);

	this->GUI->GetCheckBox(CHB_ONE)->SetLocation((int)(3*factorX), (int)(83*factorY));
	this->GUI->GetCheckBox(CHB_ONE)->SetSize((int)(21*factorX), (int)(3*factorY));
	this->GUI->GetCheckBox(CHB_ONE)->GetElement(0)->iFont = 0;
	this->GUI->GetCheckBox(CHB_ONE)->GetElement(1)->iFont = 0;
	this->GUI->GetCheckBox(CHB_ONE)->SetVisible(true);
	this->GUI->GetCheckBox(CHB_ONE)->SetChecked(this->useTextures);

	this->GUI->GetCheckBox(CHB_TWO)->SetLocation((int)(3*factorX), (int)(87*factorY));
	this->GUI->GetCheckBox(CHB_TWO)->SetSize((int)(21*factorX), (int)(3*factorY));
	this->GUI->GetCheckBox(CHB_TWO)->GetElement(0)->iFont = 0;
	this->GUI->GetCheckBox(CHB_TWO)->GetElement(1)->iFont = 0;
	this->GUI->GetCheckBox(CHB_TWO)->SetVisible(true);
	this->GUI->GetCheckBox(CHB_TWO)->SetChecked(this->showDebugInformation);

	//edit statics:
	this->GUI->GetStatic( STA_ZERO )->SetLocation((int)(3*factorX), (int)(3*factorY));
	this->GUI->GetStatic( STA_ZERO )->SetSize((int)(21*factorX), (int)(5*factorY));
	this->GUI->GetStatic( STA_ZERO )->SetTextColor( D3DCOLOR_ARGB( 255, 255, 255, 255 ) );
    this->GUI->GetControl( STA_ZERO )->GetElement( 0 )->dwTextFormat = DT_LEFT|DT_VCENTER|DT_WORDBREAK;
	this->GUI->GetControl( STA_ZERO )->GetElement( 0 )->iFont = 0;
    
	this->GUI->GetStatic( STA_ONE )->SetLocation((int)(3*factorX), (int)(7*factorY));
	this->GUI->GetStatic( STA_ONE )->SetSize((int)(21*factorX), (int)(5*factorY));
	this->GUI->GetStatic( STA_ONE )->SetTextColor( D3DCOLOR_ARGB( 255, 255, 255, 255 ) );
    this->GUI->GetControl( STA_ONE )->GetElement( 0 )->dwTextFormat = DT_LEFT|DT_VCENTER|DT_WORDBREAK;
	this->GUI->GetControl( STA_ONE )->GetElement( 0 )->iFont = 0;

	this->GUI->GetStatic( STA_TWO )->SetLocation((int)(3*factorX), (int)(57*factorY));
	this->GUI->GetStatic( STA_TWO )->SetSize((int)(21*factorX), (int)(5*factorY));
	this->GUI->GetStatic( STA_TWO )->SetTextColor( D3DCOLOR_ARGB( 255, 255, 255, 255 ) );
    this->GUI->GetControl( STA_TWO )->GetElement( 0 )->dwTextFormat = DT_LEFT|DT_VCENTER|DT_WORDBREAK;
	this->GUI->GetControl( STA_TWO )->GetElement( 0 )->iFont = 0;
	
	this->GUI->GetStatic( STA_THREE )->SetLocation((int)(3*factorX), (int)(22*factorY));
	this->GUI->GetStatic( STA_THREE )->SetSize((int)(21*factorX), (int)(5*factorY));
	this->GUI->GetStatic( STA_THREE )->SetTextColor( D3DCOLOR_ARGB( 255, 255, 255, 255 ) );
    this->GUI->GetControl( STA_THREE )->GetElement( 0 )->dwTextFormat = DT_LEFT|DT_VCENTER|DT_WORDBREAK;
	this->GUI->GetControl( STA_THREE )->GetElement( 0 )->iFont = 0;

	this->GUI->GetStatic( STA_FOUR )->SetLocation((int)(3*factorX), (int)(29*factorY));
	this->GUI->GetStatic( STA_FOUR )->SetSize((int)(21*factorX), (int)(5*factorY));
	this->GUI->GetStatic( STA_FOUR )->SetTextColor( D3DCOLOR_ARGB( 255, 255, 255, 255 ) );
    this->GUI->GetControl( STA_FOUR )->GetElement( 0 )->dwTextFormat = DT_LEFT|DT_VCENTER|DT_WORDBREAK;
	this->GUI->GetControl( STA_FOUR )->GetElement( 0 )->iFont = 0;

	this->GUI->GetStatic( STA_FIVE )->SetLocation((int)(3*factorX), (int)(11*factorY));
	this->GUI->GetStatic( STA_FIVE )->SetSize((int)(21*factorX), (int)(5*factorY));
	this->GUI->GetStatic( STA_FIVE )->SetTextColor( D3DCOLOR_ARGB( 255, 255, 255, 255 ) );
    this->GUI->GetControl( STA_FIVE )->GetElement( 0 )->dwTextFormat = DT_LEFT|DT_VCENTER|DT_WORDBREAK;
	this->GUI->GetControl( STA_FIVE )->GetElement( 0 )->iFont = 0;

	this->GUI->GetStatic( STA_SIX )->SetLocation((int)(3*factorX), (int)(36*factorY));
	this->GUI->GetStatic( STA_SIX )->SetSize((int)(21*factorX), (int)(5*factorY));
	this->GUI->GetStatic( STA_SIX )->SetTextColor( D3DCOLOR_ARGB( 255, 255, 255, 255 ) );
    this->GUI->GetControl( STA_SIX )->GetElement( 0 )->dwTextFormat = DT_LEFT|DT_VCENTER|DT_WORDBREAK;
	this->GUI->GetControl( STA_SIX )->GetElement( 0 )->iFont = 0;

	this->GUI->GetStatic( STA_SEVEN )->SetLocation((int)(3*factorX), (int)(48*factorY));
	this->GUI->GetStatic( STA_SEVEN )->SetSize((int)(21*factorX), (int)(5*factorY));
	this->GUI->GetStatic( STA_SEVEN )->SetTextColor( D3DCOLOR_ARGB( 255, 255, 255, 255 ) );
    this->GUI->GetControl( STA_SEVEN )->GetElement( 0 )->dwTextFormat = DT_LEFT|DT_VCENTER|DT_WORDBREAK;
	this->GUI->GetControl( STA_SEVEN )->GetElement( 0 )->iFont = 0;

	this->GUI->GetStatic( STA_EIGHT )->SetLocation((int)(3*factorX), (int)(15*factorY));
	this->GUI->GetStatic( STA_EIGHT )->SetSize((int)(21*factorX), (int)(5*factorY));
	this->GUI->GetStatic( STA_EIGHT )->SetTextColor( D3DCOLOR_ARGB( 255, 255, 255, 255 ) );
    this->GUI->GetControl( STA_EIGHT )->GetElement( 0 )->dwTextFormat = DT_LEFT|DT_VCENTER|DT_WORDBREAK;
	this->GUI->GetControl( STA_EIGHT )->GetElement( 0 )->iFont = 0;

	this->GUI->GetStatic( STA_NINE )->SetLocation((int)(3*factorX), (int)(66*factorY));
	this->GUI->GetStatic( STA_NINE )->SetSize((int)(21*factorX), (int)(5*factorY));
	this->GUI->GetStatic( STA_NINE )->SetTextColor( D3DCOLOR_ARGB( 255, 255, 255, 255 ) );
    this->GUI->GetControl( STA_NINE )->GetElement( 0 )->dwTextFormat = DT_LEFT|DT_VCENTER|DT_WORDBREAK;
	this->GUI->GetControl( STA_NINE )->GetElement( 0 )->iFont = 0;

	this->GUI->GetStatic( STA_TEN )->SetLocation((int)(3*factorX), (int)(73*factorY));
	this->GUI->GetStatic( STA_TEN )->SetSize((int)(21*factorX), (int)(5*factorY));
	this->GUI->GetStatic( STA_TEN )->SetTextColor( D3DCOLOR_ARGB( 255, 255, 255, 255 ) );
    this->GUI->GetControl( STA_TEN )->GetElement( 0 )->dwTextFormat = DT_LEFT|DT_VCENTER|DT_WORDBREAK;
	this->GUI->GetControl( STA_TEN )->GetElement( 0 )->iFont = 0;
}

void SoftShadowScene::setDialogResourceManager(CDXUTDialogResourceManager * DialogResourceManager) {
	this->DialogResourceManager = DialogResourceManager;
}

void SoftShadowScene::setGUI(CDXUTDialog * GUI) {
	this->GUI = GUI;
}

void SoftShadowScene::OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext )
{
	switch( nControlID )
    {
        case BUT_EXIT:
			SendMessage( DXUTGetHWND(), WM_CLOSE, 0, 0 );
			break;
		case CMB_ZERO:
			{
				DXUTComboBoxItem *pItem = GUI->GetComboBox(CMB_ZERO)->GetSelectedItem();
				ComboBoxZeroChanged(pItem->pData);
				break;
			}
		case CMB_ONE:
			{
				DXUTComboBoxItem *pItem = GUI->GetComboBox(CMB_ONE)->GetSelectedItem();
				ComboBoxOneChanged(pItem->pData);
				break;
			}
		case CMB_TWO:
			{
				DXUTComboBoxItem *pItem = GUI->GetComboBox(CMB_TWO)->GetSelectedItem();
				ComboBoxTwoChanged(pItem->pData);
				break;
			}
		case CHB_ZERO:
            CheckBoxZeroChanged(((CDXUTCheckBox*)pControl)->GetChecked());
            break;
		case CHB_ONE:
            CheckBoxOneChanged(((CDXUTCheckBox*)pControl)->GetChecked());
            break;
		case CHB_TWO:
            CheckBoxTwoChanged(((CDXUTCheckBox*)pControl)->GetChecked());
            break;
		case SLD_ZERO:
			SliderZeroChanged(((CDXUTSlider*)pControl)->GetValue());
			break;
		case SLD_ONE:
			SliderOneChanged(((CDXUTSlider*)pControl)->GetValue());
			break;
		case SLD_TWO:
			SliderTwoChanged(((CDXUTSlider*)pControl)->GetValue());
			break;
		case SLD_THREE:
			SliderThreeChanged(((CDXUTSlider*)pControl)->GetValue());
			break;
		case SLD_FOUR:
			SliderFourChanged(((CDXUTSlider*)pControl)->GetValue());
			break;
    }
}

void SoftShadowScene::renderGUI(float fElapsedTime) 
{
	this->device->BeginScene();
	this->GUI->OnRender(fElapsedTime);
	this->device->EndScene();

	LowMemLight* currentLight = this->getCurrentLight();

	size_t count = 100;

	WCHAR tempChar[200];
	swprintf(tempChar, count, L"Number of depth maps: %i", currentLight->getNbShadowMaps());
	this->GUI->GetStatic( STA_FIVE )->SetText(tempChar);

	swprintf(tempChar, count, L"FPS: %i", (int)DXUTGetFPS());
	this->GUI->GetStatic( STA_ONE )->SetText(tempChar);

	swprintf(tempChar, count, L"Epsilon: %f", (float)currentLight->getEpsilon());
	this->GUI->GetStatic( STA_THREE )->SetText(tempChar);

	swprintf(tempChar, count, L"Max. depth diff.: %f", (float)currentLight->getErrorThreshold());
	this->GUI->GetStatic( STA_FOUR )->SetText(tempChar);

	swprintf(tempChar, count, L"Shadow map bias: %f", (float)currentLight->getSMBias());
	this->GUI->GetStatic( STA_SIX )->SetText(tempChar);

	swprintf(tempChar, count, L"Light size: %f", (float)currentLight->getLightDimension());
	this->GUI->GetStatic( STA_EIGHT )->SetText(tempChar);

	swprintf(tempChar, count, L"Sampling distance: %f", (float)currentLight->getFilterSamplingDistance());
	this->GUI->GetStatic( STA_NINE )->SetText(tempChar);


	if(bFirstGUIRendering) {
		this->GUI->GetSlider(SLD_ZERO)->SetValue((int)((this->currentLight->getEpsilon() / (MAX_EPSILON - MIN_EPSILON))*100));
		this->GUI->GetSlider(SLD_ONE)->SetValue((int)((this->currentLight->getErrorThreshold() / (MAX_ABS - MIN_ABS))*100));
		this->GUI->GetSlider(SLD_TWO)->SetValue((int)((this->currentLight->getSMBias() / (MAX_BIAS - MIN_BIAS))*100));
		this->GUI->GetSlider(SLD_THREE)->SetValue((int)((this->currentLight->getFilterSamplingDistance() / (MAX_SAMPLINGDIST - MIN_SAMPLINGDIST))*100));
		this->GUI->GetSlider(SLD_FOUR)->SetValue((int)((this->currentLight->getLightDimension() / (MAX_LIGHTDIM - MIN_LIGHTDIM))*100));

		bFirstGUIRendering = false;
	}
}

void SoftShadowScene::switchGUI() {
	this->bGUIVisible = !this->bGUIVisible;
	this->GUI->EnableKeyboardInput(this->bGUIVisible);
	this->GUI->EnableMouseInput(this->bGUIVisible);
}

void SoftShadowScene::SliderZeroChanged(int iValue) {
	this->GUI->GetSlider(SLD_ZERO)->SetValue(iValue);

	float dx = (MAX_EPSILON - MIN_EPSILON) / 100;
	this->currentLight->setEpsilon(MIN_EPSILON + iValue * dx);
}

void SoftShadowScene::SliderOneChanged(int iValue) {
	this->GUI->GetSlider(SLD_ONE)->SetValue(iValue);

	float dx = (MAX_ABS - MIN_ABS) / 100;
	this->currentLight->setErrorThreshold(MIN_ABS + iValue * dx);
}

void SoftShadowScene::SliderTwoChanged(int iValue) {
	this->GUI->GetSlider(SLD_TWO)->SetValue(iValue);

	float dx = (MAX_BIAS - MIN_BIAS) / 100;
	this->currentLight->setSMBias(MIN_BIAS + iValue * dx);
}

void SoftShadowScene::SliderThreeChanged(int iValue) {
	this->GUI->GetSlider(SLD_THREE)->SetValue(iValue);

	float dx = (MAX_SAMPLINGDIST - MIN_SAMPLINGDIST) / 100;
	this->currentLight->setFilterSamplingDistance(MIN_SAMPLINGDIST + iValue * dx);
}

void SoftShadowScene::SliderFourChanged(int iValue) {
	this->GUI->GetSlider(SLD_FOUR)->SetValue(iValue);

	float dx = (MAX_LIGHTDIM - MIN_LIGHTDIM) / 100;
	this->currentLight->setLightDimension(MIN_LIGHTDIM + iValue * dx);
}

void SoftShadowScene::ComboBoxZeroChanged(void *Value) {
	int iNewKernelSize = 0;

	if (Value == (LPVOID)0) {
		iNewKernelSize = 3;
	} else if (Value == (LPVOID)1) {
		iNewKernelSize = 5;
	} else if (Value == (LPVOID)2) {
		iNewKernelSize = 7;
	} else if (Value == (LPVOID)3) {
		iNewKernelSize = 9;
	} 

	this->currentLight->setFilterKernelSize(iNewKernelSize);
	this->currentLight->kernelSizeChanged = true;
}

void SoftShadowScene::ComboBoxOneChanged(void *Value) {
	if (Value == (LPVOID)0) {
		this->filterMode = 0;
	} else if (Value == (LPVOID)1) {
		this->filterMode = 1;
	} else if (Value == (LPVOID)2) {
		this->filterMode = 2;
	} 
}

void SoftShadowScene::ComboBoxTwoChanged(void *Value) {
	if (Value == (LPVOID)0) {
		this->shadowingMode = 0;
	} else if (Value == (LPVOID)1) {
		this->shadowingMode = 1;
	} else if (Value == (LPVOID)2) {
		this->shadowingMode = 2;
	} 
}

void SoftShadowScene::CheckBoxZeroChanged(bool bValue) {
	if(bValue) {
		this->viewPointMode = Scene::VIEWPOINT_CAMERA;
	} else {
		this->viewPointMode = Scene::VIEWPOINT_LIGHT;
	}
}

void SoftShadowScene::CheckBoxOneChanged(bool bValue) {
	this->useTextures = bValue;
}

void SoftShadowScene::CheckBoxTwoChanged(bool bValue) {
	this->showDebugInformation = bValue;
}