#pragma once
#include "scene.h"
#include <string>
#include <list>
#include <vector>
#include "fmod.h"

#define SPTR boost::shared_ptr

class SkyBox;
class SoundNode;
class ShadowSceneLoader;
class LowMemLight;

/**
SoftShadowScene class
*/

class SoftShadowScene :
	public Scene
{
public:
	friend class ShadowSceneLoader;

	/**
	Constructor for SoftShadowScene object
	*/
	SoftShadowScene(void);
	/**
	Destructor for SoftShadowScene object
	*/
	~SoftShadowScene(void);

	/**
	SoftShadowScene initialization
	*/
	virtual void initScene(GameManager &_manager);
	/**
	clear SoftShadowScene data
	*/
	virtual void clearScene();
	/**
	Render SoftShadowScene
	*/
	virtual void renderScene(float fElapsedTime);
	
	/**
	SkyBox object
	*/
	SkyBox *skybox;

	virtual Node* createNode(int type);
	virtual Node* createNode(int type, Node &father);
	virtual Node* createNode(int type, bool addDefaultRenderer);
	virtual Node* createNode(int type, Node &father, bool addDefaultRenderer);
	virtual Node* createNode(int type, Node &father, bool addDefaultRenderer, bool isReference);

	//PhysicStuff
	void connectMaterialIds(int physicId, int xmlId);
	int getPhysicMaterialId(int xmlId);
	std::vector<int> materialXMLVector;
	std::vector<int> materialPhysicVector;

	//GoodieStuff
	float getRandomTime(float timeBase, float timeVariation);

	static const int TRIGGER_IMPACTPLAYER	= 100;
	static const int TRIGGER_IMPACTTERRAIN	= 101;
	static const int TRIGGER_IMPACTGOODIE	= 102;
	static const int TRIGGER_IMPACTOTHER	= 103;
	static const int TRIGGER_GOODIECATCH	= 104;
	static const int TRIGGER_IMPACTFIRE		= 105;
	static const int TRIGGER_IMPACTICE		= 106;
	static const int TRIGGER_CREATEPLAYER	= 107;
	static const int TRIGGER_CREATEMOREPLAYER= 108;
	static const int TRIGGER_CHANGECAM		= 109;

	static const int TRIGGER_EN_SHADOW		= 110;
	static const int TRIGGER_DIS_SHADOW		= 111;
	static const int TRIGGER_EN_CARTOON		= 112;
	static const int TRIGGER_DIS_CARTOON	= 113;
	static const int TRIGGER_EN_MOTION		= 114;
	static const int TRIGGER_DIS_MOTION		= 115;

	static const int TRIGGER_MESSAGE		= 116;

	virtual void cleanUpScene();
	
	/**
	Updates the Camera depending on the user input
	*/
	void updateCamera(float dt);
	/**
	sets up the GUI
	*/
	void setupGUI();
	/**
	Setter for the GUI ResourceManager
	*/
	void setDialogResourceManager(CDXUTDialogResourceManager * DialogResourceManager);
	/**
	Setter for the GUI Object
	*/
	void setGUI(CDXUTDialog * GUI);
	/**
	Event handling for GUI elements
	*/
	virtual void OnGUIEvent( UINT nEvent, int nControlID, CDXUTControl* pControl, void* pUserContext );
	/**
	render the GUI
	*/
	void renderGUI(float fElapsedTime);
	/**
	turn GUI on/off
	*/
	void switchGUI();
	/**
	List of Objects in the scene
	*/
	std::vector<Object3d*> sceneObjectList;
	/**
	Current Scene ID
	*/
	int iCurrentSceneId;
	/**
	The Light Source
	*/
	LowMemLight* light;
protected:
	void SliderZeroChanged(int iValue);
	void SliderOneChanged(int iValue);
	void SliderTwoChanged(int iValue);
	void SliderThreeChanged(int iValue);
	void SliderFourChanged(int iValue);
	void ComboBoxZeroChanged(void *Value);
	void ComboBoxOneChanged(void *Value);
	void ComboBoxTwoChanged(void *Value);
	void CheckBoxZeroChanged(bool bValue);
	void CheckBoxOneChanged(bool bValue);
	void CheckBoxTwoChanged(bool bValue);
	bool ActionInMenu();

	Vector lookAtPoint;
	Vector viewPoint;
	Vector dLookAtPoint;
	Vector dViewPoint;

	//Reference Lists
	std::vector<SPTR<Node> > refWeaponVector;			//List of all Weapons
	std::vector<SPTR<Node> > refBulletVector;			//List of all Bullets

	virtual void executeTrigger(SPTR<Trigger> trigger);

	bool mousePressed;
	bool rightMousePressed;
	float camRadius;
	float camYaw;
	float camPitch;
	float camDesRadius;
	float camDesYaw;
	float camDesPitch;
	Vector camPos;
	float oldMousePos[2];
	float oldZoomMousePos;
	int oldWheel;
	float maxPitch;
	float camFactor;
	Vector desiredLookAtTarget;
	Vector lookAtTarget;
	float orthoDimension;

	/**
	dialog resource manager for the DirectX GUI
	*/
	CDXUTDialogResourceManager *DialogResourceManager;
	/**
	the directX GUI
	*/
	CDXUTDialog *GUI;
	/**
	boolean for GUI visibility
	*/
	bool bGUIVisible;
	/**
	boolean which indicates the first rendering of the GUI
	*/
	bool bFirstGUIRendering;

	#define BUT_EXIT				3
	
	#define EDB_BACKGROUND			20
	
	#define CMB_ZERO				30
	#define CMB_ONE					31
	#define CMB_TWO					32
		
	#define SLD_ZERO				40
	#define SLD_ONE					41
	#define SLD_TWO					42
	#define SLD_THREE				43
	#define SLD_FOUR				44
	
	#define CHB_ZERO				50
	#define CHB_ONE					51
	#define CHB_TWO					52
	
	#define STA_ZERO				70
	#define STA_ONE					71
	#define STA_TWO					72
	#define STA_THREE				73
	#define STA_FOUR				74
	#define STA_FIVE				75
	#define STA_SIX					76
	#define STA_SEVEN				77
	#define STA_EIGHT				78
	#define STA_NINE				79
	#define STA_TEN					80

	float MIN_EPSILON;
	float MAX_EPSILON;

	float MIN_ABS;
	float MAX_ABS;

	float MIN_BIAS;
	float MAX_BIAS;

	float MIN_LIGHTDIM;
	float MAX_LIGHTDIM;

	float MIN_SAMPLINGDIST;
	float MAX_SAMPLINGDIST;
};
